@@ -72,6 +72,7 @@ class classification(object):
         """Load race config from disk."""
         cr = ConfigParser.ConfigParser({'id':EVENT_ID,
                                         'showinfo':'Yes',
+                                        'showevents':'',
                                         'placesrc':'',
                                         'medals':'' })
         cr.add_section('event')
@@ -84,6 +85,7 @@ class classification(object):
         self.info_expand.set_expanded(strops.confopt_bool(
                                        cr.get('event', 'showinfo')))
 
+        self.showevents = cr.get('event', 'showevents')
         self.placesrc = cr.get('event', 'placesrc')
         self.medals = cr.get('event', 'medals')
         self.recalculate()	# model is cleared and loaded in recalc
@@ -128,6 +130,7 @@ class classification(object):
             return
         cw = ConfigParser.ConfigParser()
         cw.add_section('event')
+        cw.set('event', 'showevents', self.showevents)
         cw.set('event', 'placesrc', self.placesrc)
         cw.set('event', 'medals', self.medals)
         cw.set('event', 'showinfo', self.info_expand.get_expanded())
@@ -157,6 +160,8 @@ class classification(object):
     def result_report(self):
         """Return a list of printing sections containing the race result."""
         ret = []
+
+        # start with the overall result
         sec = printing.section()
         sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
                        self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
@@ -185,6 +190,21 @@ class classification(object):
             nrow = [rank, rno, rname, None, None, medal]
             sec.lines.append(nrow)
         ret.append(sec)
+
+        # then append each of the specified events
+        for evno in self.showevents.split():
+            if evno:
+                self.log.info('looking for event: ' + repr(evno))
+                r = self.meet.get_event(evno, False)
+                if r is None:
+                    self.log.error('Invalid event in showplaces.')
+                    continue
+                r.loadconfig()	# now have queryable event handle
+                if r.onestart:	# go for result
+                    ret.extend(r.result_report())
+                else:		# go for startlist
+                    ret.append(r.startlist_report())
+                r.destroy()
         return ret
 
     def result_export(self, f):
@@ -409,6 +429,8 @@ class classification(object):
         dlg.set_transient_for(self.meet.window)
         se = b.get_object('race_series_entry')
         se.set_text(self.series)
+        ee = b.get_object('race_showevents_entry')
+        ee.set_text(self.showevents)
         pe = b.get_object('race_placesrc_entry')
         pe.set_text(self.placesrc)
         me = b.get_object('race_medals_entry')
@@ -419,6 +441,7 @@ class classification(object):
             # update event config
             self.placesrc = pe.get_text()
             self.medals = me.get_text()
+            self.showevents = ee.get_text()
 
             # update series
             ns = se.get_text()
