@@ -110,7 +110,7 @@ class eventdb(object):
                     if len(ir) > 0 and ir[COL_EVNO] != 'Num':
                         num = ir[COL_EVNO].strip()
                         if strops.reformat_bibserlist(num) == num:
-                            nr = [num, '', '', '', '', False, '', '', True]
+                            nr = [num, '', '', '', '', False, '', '', False]
                             for i in range(1,8):
                                 if len(ir) > i:
                                     if i != COL_OPEN:
