@@ -389,6 +389,7 @@ class ittt(object):
         ret = []
         cnt = 0
         sec = printing.dual_ittt_startlist()
+        sec.showheats = True
         if self.timetype == 'single':
             sec.set_single()
 
@@ -464,6 +465,8 @@ class ittt(object):
             lane = 1
             for r in self.riders:
                 rno = r[COL_BIB]
+                if 't' in self.series:	# Team no hack
+                    rno = ' '	# force name
                 rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
                                    r[COL_CLUB])
                 heat = str(hno) + '.' + str(lane)
@@ -745,6 +748,8 @@ class ittt(object):
                     else:
                         dtime = '+' + (time - ftime).rawtime(2)
                     rtime = time.rawtime(3)
+                elif r[COL_COMMENT]:
+                    rtime = str(r[COL_COMMENT])
 
             sec.lines.append([rank, rno, rname, None, rtime, dtime])
         ret.append(sec)
