@@ -110,6 +110,15 @@ XS_TITLE = xlwt.easyxf('font: bold on')
 XS_SUBTITLE = xlwt.easyxf('font: italic on')
 XS_MONOSPACE = xlwt.easyxf('font: name Courier')
 
+# Meta cell icon classes
+ICONMAP = {'datestr':'icon-search',
+           'docstr':'icon-flag',
+           'diststr':'icon-road', 
+           'commstr':'icon-user',
+           'orgstr':'icon-star',
+           'download':'icon-file',
+           'default':'icon-file'}
+
 # conversions
 def pt2mm(pt=1):
     """72pt -> 25.4mm (1 inch)"""
@@ -131,15 +140,16 @@ def pt2pt(pt=1):
     """Dummy conversion."""
     return pt
 
-BODYFONT = 'TeXGyreTermes 7.4'			# body text
-BODYBOLDFONT = 'TeXGyreTermes bold 7.4'		# bold body text
-MONOSPACEFONT = 'TexGyreCursor bold 7.4'	# monospaced text
-SECTIONFONT = 'TeXGyreAdventor bold 7.4'	# section headings
-SUBHEADFONT = 'TeXGyreTermes italic 7.4'	# section subheadings
+FEPSILON = 0.0001				# float epsilon
+BODYFONT = 'TeXGyreTermes 8.0'			# body text
+BODYBOLDFONT = 'TeXGyreTermes bold 8.0'		# bold body text
+MONOSPACEFONT = 'TeXGyreCursor bold 7.4'	# monospaced text
+SECTIONFONT = 'TeXGyreHeros bold 8.0'		# section headings
+SUBHEADFONT = 'TeXGyreTermes italic 8.0'	# section subheadings
 PROVFONT = 'Nimbus Sans L Bold Condensed 90'	# provisonal underlay
-LINE_HEIGHT = mm2pt(5.0)			# body text line height
-PAGE_OVERFLOW = mm2pt(2.0)			# section overflow
-SECTION_HEIGHT = mm2pt(7.0)			# height of section title
+LINE_HEIGHT = mm2pt(5.5)			# body text line height
+PAGE_OVERFLOW = mm2pt(3.0)			# section overflow
+SECTION_HEIGHT = mm2pt(6.5)			# height of section title
 
 
 UNITSMAP = { 'mm':mm2pt,
@@ -270,16 +280,31 @@ def vec2htmlrow(vec=[]):
     cols = []
     cols.append(htlib.td(htlib.escapetext(rowmap[0])))	# Rank (left)
     cols.append(htlib.td(htlib.escapetext(rowmap[1]),
-                           {'align':'right'}))	# No (right)
+                           {'class':'right'}))	# No (right)
     cols.append(htlib.td(htlib.escapetext(rowmap[2])))	# Name (left)
     cols.append(htlib.td(htlib.escapetext(rowmap[3])))	# Cat/Code (left)
     cols.append(htlib.td(htlib.escapetext(rowmap[4]),
-                           {'align':'right'}))	# time/gap (right)
+                           {'class':'right'}))	# time/gap (right)
     cols.append(htlib.td(htlib.escapetext(rowmap[5]),
-                           {'align':'right'}))	# time/gap (right)
+                           {'class':'right'}))	# time/gap (right)
     cols.append(htlib.td(htlib.escapetext(rowmap[6])))	# Units (left)
     return htlib.tr(cols)
 
+def vec2htmlhead(vec=[]):
+    rowmap = vecmapstr(vec, 7)
+    cols = []
+    cols.append(htlib.th(htlib.escapetext(rowmap[0])))	# Rank (left)
+    cols.append(htlib.th(htlib.escapetext(rowmap[1]),
+                           {'class':'right'}))	# No (right)
+    cols.append(htlib.th(htlib.escapetext(rowmap[2])))	# Name (left)
+    cols.append(htlib.th(htlib.escapetext(rowmap[3])))	# Cat/Code (left)
+    cols.append(htlib.th(htlib.escapetext(rowmap[4]),
+                           {'class':'right'}))	# time/gap (right)
+    cols.append(htlib.th(htlib.escapetext(rowmap[5]),
+                           {'class':'right'}))	# time/gap (right)
+    cols.append(htlib.th(htlib.escapetext(rowmap[6])))	# Units (left)
+    return htlib.tr(cols)
+
 def vec2line(vec=[]):
     ret = []
     for i in vec:
@@ -293,42 +318,42 @@ class dual_ittt_startlist(object):
     """Two-up time trial for individual riders."""
     def __init__(self):
         self.heading = None
+        self.subheading = None
+        self.recordstr = None
+        self.showheats = False	# show heat labels?
         self.lines = []
         self.fslbl = 'Front Straight'
         self.bslbl = 'Back Straight'
-        self.lenstr = ''
-        self.xtra1 = ''
-        self.xtra2 = ''
         self.lcount = 0
-        self.recordstr = ''
         self.h = None
         
     def set_record(self, recstr):
-        """Set the Australian record for this event."""
+        """Set or clear the record string for this event."""
         if recstr:
             if recstr[0].isdigit():
                 self.recordstr = 'Australian Record: ' + recstr
             else:	# ASSUME prompt provided
                 self.recordstr = recstr
         else:
-            self.recordstr = ''
+            self.recordstr = None
 
     def set_single(self):
         """Convenience func to make 'single lane'."""
         self.fslbl = ''
         self.bslbl = ''
+        self.showheats = False
 
     def get_h(self, report):
         """Return total height on page of section on report."""
         if self.h is None or len(self.lines) != self.lcount:
             self.h = report.line_height * len(self.lines)
-            if self.bslbl:
+            if self.showheats:	# if heats are shown, double line height
                 self.h *= 2
             if self.heading:
                 self.h += report.section_height
-            if self.fslbl or self.bslbl:
+            if self.subheading:
                 self.h += report.line_height
-            if self.xtra1 or self.xtra2:
+            if self.fslbl or self.bslbl:
                 self.h += report.line_height
             if self.recordstr:
                 self.h += report.line_height
@@ -336,11 +361,15 @@ class dual_ittt_startlist(object):
         return self.h
 
     def truncate(self, remainder, report):
-        # Fit whole section on page
+        """Return a copy of the section up to page break."""
+        # program event sections do not break ... 
         if self.get_h(report) <= (remainder + report.page_overflow):
             return (self, None)
         else:
-            # move section onto next page
+            if report.pagefrac() < FEPSILON:	# avoid error
+                raise RuntimeWarning('Section ' + repr(self.heading)
+                          + ' will not fit on a page and will not break.')
+            # move entire section onto next page
             return (pagebreak(), self)
 
     def draw_pdf(self, report):
@@ -350,43 +379,30 @@ class dual_ittt_startlist(object):
             report.text_cent(report.midpagew, report.h, self.heading,
                            report.fonts['section'])
             report.h += report.section_height
-        subheadvec = ['', '', '']
-        dosubhead = False
-        if self.lenstr:
-            subheadvec[0] = self.lenstr
-            dosubhead = True
-        if self.xtra2:
-            subheadvec[1] = self.xtra2
-            dosubhead = True
-        if self.xtra1:
-            subheadvec[2] = self.xtra1
-            dosubhead = True
-        if dosubhead:
-            report.text_cent(report.midpagew, report.h, 
-                       '\t'.join(subheadvec).strip(), report.fonts['subhead'])
+        if self.subheading:
+            report.text_cent(report.midpagew, report.h, self.subheading,
+                              report.fonts['subhead'])
             report.h += report.line_height
-        dosubhead = False
+        dolanes = False
         dual = False
         if self.fslbl:
             report.text_cent(report.midpagew-mm2pt(40), report.h, self.fslbl,
                               report.fonts['subhead'])
-            dosubhead = True
+            dolanes = True
         if self.bslbl:
             report.text_left(report.midpagew+mm2pt(40), report.h, self.bslbl,
                               report.fonts['subhead'])
-            dosubhead = True
-            dual = True
-        if dosubhead:
-            report.h += report.line_height
-
+            dolanes = True
+            dual = True		# heading flags presense of back straight
+        if dolanes:
+            report.h += report.line_height # account for lane label h
         hof = report.h
         lineheight = report.line_height
-        if dual:
+        if self.showheats:
             lineheight *= 2
         for i in self.lines:
-            report.ittt_heat(i, hof, dual)
+            report.ittt_heat(i, hof, dual, self.showheats)
             hof += lineheight
-        
         if self.recordstr:
             report.text_cent(report.midpagew, hof, self.recordstr,
                               report.fonts['subhead'])
@@ -396,11 +412,91 @@ class dual_ittt_startlist(object):
 
     def draw_xls(self, report, worksheet):
         """Output program element to excel worksheet."""
+        row = report.h
+        if self.heading:
+            worksheet.write(row, 2, self.heading.strip(), XS_TITLE)
+            row += 1
+        if self.subheading:
+            worksheet.write(row, 2,
+                 self.subheading.replace('\t', '  ').strip(), XS_SUBTITLE)
+            row += 2
+        else:
+            row += 1	# min one clear row between
+        dual = False
+        if self.bslbl:
+            dual = True
+        if len(self.lines) > 0:
+            rows = []
+            for r in self.lines:
+                nv = [None, None, None]
+                if self.showheats and r[0] and r[0] != '-':
+                    nv[0] = 'Heat ' + str(r[0])
+                if len(r) > 3:	# front straight
+                    nv[1] = r[1]
+                    nv[2] = r[2]
+                rows.append(nv)
+                if len(r) > 7:	# back straight
+                    nv = [None, r[5], r[6]]
+                    rows.append(nv)
+                elif dual:
+                    rows.append([None, None, '[No Rider]'])
+            for rw in rows:
+                l = vecmapstr(rw)
+                # todo: apply styles to whole doc?
+                worksheet.write(row, 0, l[0], XS_LEFT)
+                worksheet.write(row, 1, l[1], XS_RIGHT)
+                worksheet.write(row, 2, l[2], XS_LEFT)
+                worksheet.write(row, 3, l[3], XS_LEFT)
+                worksheet.write(row, 4, l[4], XS_RIGHT)
+                worksheet.write(row, 5, l[5], XS_RIGHT)
+                worksheet.write(row, 6, l[6], XS_LEFT)
+                row += 1
+            row += 1
+        if self.recordstr:
+            worksheet.write(row, 2, self.recordstr.strip(), XS_SUBTITLE)
+            row += 2
+        #else:
+            #row += 1	# min one clear row between
+        report.h = row
         return None
 
     def draw_text(self, report, f):
         """Output program element in markdown."""
-        return ''
+        if self.heading:
+            f.write('### ' + self.heading.strip() + '\n\n')
+        if self.subheading:
+            f.write(htlib.p(htlib.escapetext(self.subheading.strip()), 
+                            {'class':'lead'}) + '\n\n')
+        dual = False
+        if self.bslbl:
+            dual = True
+        if len(self.lines) > 0:
+            rows = []
+            for r in self.lines:
+                nv = [None, None, None]
+                if self.showheats and r[0] and r[0] != '-':
+                    nv[0] = 'Heat ' + str(r[0]) + ':'
+                if len(r) > 3:	# front straight
+                    nv[1] = r[1]
+                    nv[2] = r[2]
+                rows.append(nv)
+                if len(r) > 7:	# back straight
+                    nv = [None, r[5], r[6]]
+                    rows.append(nv)
+                elif dual:
+                    rows.append([None, None, '[No Rider]'])
+
+            trows = []
+            for l in rows:
+                trows.append(vec2htmlrow(l))
+            f.write(htlib.table(htlib.tbody(trows),
+                  {'class':'table table-striped table-condensed',
+                   'style':'width: auto'}))
+            f.write('\n\n')
+
+        if self.recordstr:
+            f.write(self.recordstr.strip() + '\n\n')
+        return False
 
 class twocol_startlist(object):
     def __init__(self):
@@ -430,7 +526,7 @@ class twocol_startlist(object):
         if self.get_h(report) <= (remainder + report.page_overflow):
             return (self, None)
         else:
-            if report.pagefrac() < 0.0001:	# avoid error
+            if report.pagefrac() < FEPSILON:	# avoid error
                 raise RuntimeWarning('Section ' + repr(self.heading)
                           + ' will not fit on a page and will not break.')
             # move entire section onto next page
@@ -471,7 +567,6 @@ class twocol_startlist(object):
             report.drawline(report.body_right - mm2pt(20.0), baseline,
                             report.body_right, baseline)
             report.h += report.line_height
-
         report.c.restore()
 
     def draw_xls(self, report, worksheet):
@@ -532,29 +627,31 @@ class twocol_startlist(object):
 class sprintround(object):
     def __init__(self):
         self.heading = None
-        self.lenstr = None
-        self.xtra1 = None
-        self.xtra2 = None
-        self.lines = []		 # maps to 'heats'
+        self.subheading = None
+        self.lines = []		 # maps to 'heats', include riders?
         self.lcount = 0
         self.h = None
         
     def get_h(self, report):
         """Return total height on page of section on report."""
         if self.h is None or len(self.lines) != self.lcount:
-            self.h = report.line_height * len(self.lines)
+            self.h = report.line_height * len(self.lines) # one per line?
             if self.heading:
                 self.h += report.section_height
-            if self.xtra1 or self.xtra2 or self.lenstr:
+            if self.subheading:
                 self.h += report.line_height
             self.lcount = len(self.lines)
         return self.h
 
     def truncate(self, remainder, report):
-        # program event sections do not break
+        """Return a copy of the section up to page break."""
+        # program event sections do not break ... 
         if self.get_h(report) <= (remainder + report.page_overflow):
             return (self, None)
         else:
+            if report.pagefrac() < FEPSILON:	# avoid error
+                raise RuntimeWarning('Section ' + repr(self.heading)
+                          + ' will not fit on a page and will not break.')
             # move entire section onto next page
             return (pagebreak(), self)
 
@@ -565,22 +662,10 @@ class sprintround(object):
             report.text_cent(report.midpagew, report.h, self.heading,
                            report.fonts['section'])
             report.h += report.section_height
-        subheadvec = ['', '', '']
-        dosubhead = False
-        if self.lenstr:
-            subheadvec[0] = self.lenstr
-            dosubhead = True
-        if self.xtra2:
-            subheadvec[1] = self.xtra2
-            dosubhead = True
-        if self.xtra1:
-            subheadvec[2] = self.xtra1
-            dosubhead = True
-        if dosubhead:
-            report.text_cent(report.midpagew, report.h, 
-                       '\t'.join(subheadvec).strip(), report.fonts['subhead'])
+        if self.subheading is not None:
+            report.text_cent(report.midpagew, report.h, self.subheading,
+                           report.fonts['subhead'])
             report.h += report.line_height
-
         hof = report.h
         if len(self.lines) > 0:
             for i in self.lines:
@@ -590,16 +675,19 @@ class sprintround(object):
                 if heat:
                     report.text_left(report.body_left, hof,
                                      heat, report.fonts['subhead']) 
-                report.rms_rider(i[1], report.body_left + mm2pt(12), hof)
-                report.text_cent(report.midpagew - mm2pt(2), hof, 'v',
-                                  report.fonts['subhead']) 
-                report.rms_rider(i[2], report.midpagew + mm2pt(2), hof)
+                report.sprint_rider(i[1], report.body_left + mm2pt(14), hof)
+                report.sprint_rider(i[2], report.midpagew + mm2pt(4), hof)
+                vstr = 'v'
+                if i[1][0] and i[2][0]:	# assume result in order...
+                    vstr = 'def'
+                report.text_cent(report.midpagew, hof,
+                                 vstr, report.fonts['subhead']) 
                 time = ''
-                if i[3]:
-                    time = i[3]
+                if len(i) > 3 and i[3]:
+                    time = i[3]	# probably already have a result
                 if time:
-                    report.text_left(report.body_right - mm2pt(10), hof,
-                                     time, report.fonts['body']) 
+                    report.text_right(report.body_right, hof,
+                                      time, report.fonts['body']) 
                 else:
                     baseline = report.get_baseline(hof)
                     report.drawline(report.body_right - mm2pt(10),
@@ -612,18 +700,89 @@ class sprintround(object):
 
     def draw_xls(self, report, worksheet):
         """Output program element to excel worksheet."""
+        row = report.h
+        if self.heading:
+            worksheet.write(row, 2, self.heading.strip(), XS_TITLE)
+            row += 1
+        if self.subheading:
+            worksheet.write(row, 2,
+                 self.subheading.replace('\t', '  ').strip(), XS_SUBTITLE)
+            row += 2
+        else:
+            row += 1	# min one clear row between
+        if len(self.lines) > 0:
+            rows = []
+            for c in self.lines:	# each row is a pair/contest
+                # 'a' rider
+                rows.append([None, None, c[0], None, None])	# contest id)
+                av = [None, None, None, None, None]
+                av[0] = c[1][0]
+                av[1] = c[1][1]
+                av[2] = c[1][2]
+                av[3] = c[1][3]
+                if len(c) > 3 and c[3]:
+                    av[4] = c[3]	# place 200m time in info col
+                rows.append(av)
+                # 'b' rider
+                bv = [None, None, None, None, None]
+                bv[0] = c[2][0]
+                bv[1] = c[2][1]
+                bv[2] = c[2][2]
+                bv[3] = c[2][3]
+                rows.append(bv)
+            for rw in rows:
+                l = vecmapstr(rw)
+                # todo: apply styles to whole doc?
+                worksheet.write(row, 0, l[0], XS_LEFT)
+                worksheet.write(row, 1, l[1], XS_RIGHT)
+                worksheet.write(row, 2, l[2], XS_LEFT)
+                worksheet.write(row, 3, l[3], XS_LEFT)
+                worksheet.write(row, 4, l[4], XS_RIGHT)
+                worksheet.write(row, 5, l[5], XS_RIGHT)
+                worksheet.write(row, 6, l[6], XS_LEFT)
+                row += 1
+            row += 1
+        report.h = row
         return None
 
     def draw_text(self, report, f):
         """Output program element in markdown."""
+        if self.heading:
+            f.write('### ' + self.heading.strip() + '\n\n')
+        if self.subheading:
+            f.write('#### ' + self.subheading.strip() + '\n\n')
+        if len(self.lines) > 0:
+            rows = []
+            for c in self.lines:	# each row is a pair/contest
+                # 'a' rider
+                rows.append([None, None, c[0], None, None])	# contest id)
+                av = [None, None, None, None, None]
+                av[0] = c[1][0]
+                av[1] = c[1][1]
+                av[2] = c[1][2]
+                av[3] = c[1][3]
+                if len(c) > 3 and c[3]:
+                    av[4] = c[3]	# place 200m time in info col
+                rows.append(av)
+                # 'b' rider
+                bv = [None, None, None, None, None]
+                bv[0] = c[2][0]
+                bv[1] = c[2][1]
+                bv[2] = c[2][2]
+                bv[3] = c[2][3]
+                rows.append(bv)
+            trows = []
+            for l in rows:
+                trows.append(vec2htmlrow(l))
+            f.write(htlib.table(trows, {'border':'0',
+                                 'cellspacing':'2', 'cellpadding':'2'}))
+            f.write('\n\n')
         return ''
 
 class sprintfinal(object):
     def __init__(self):
         self.heading = None
-        self.lenstr = None
-        self.xtra1 = None
-        self.xtra2 = None
+        self.subheading = None
         self.lines = []		 # maps to 'contests'
         self.lcount = 0
         self.h = None
@@ -634,17 +793,20 @@ class sprintfinal(object):
             self.h = report.line_height * 3.0 * len(self.lines)
             if self.heading:
                 self.h += report.section_height
-            if self.xtra1 or self.xtra2 or self.lenstr:
+            if self.subheading:
                 self.h += report.line_height
             self.lcount = len(self.lines)
         return self.h
 
     def truncate(self, remainder, report):
         """Return a copy of the section up to page break."""
-        # program event sections do not break
+        # program event sections do not break ... 
         if self.get_h(report) <= (remainder + report.page_overflow):
             return (self, None)
         else:
+            if report.pagefrac() < FEPSILON:	# avoid error
+                raise RuntimeWarning('Section ' + repr(self.heading)
+                          + ' will not fit on a page and will not break.')
             # move entire section onto next page
             return (pagebreak(), self)
 
@@ -655,22 +817,10 @@ class sprintfinal(object):
             report.text_cent(report.midpagew, report.h, self.heading,
                            report.fonts['section'])
             report.h += report.section_height
-        subheadvec = ['', '', '']
-        dosubhead = False
-        if self.lenstr:
-            subheadvec[0] = self.lenstr
-            dosubhead = True
-        if self.xtra2:
-            subheadvec[1] = self.xtra2
-            dosubhead = True
-        if self.xtra1:
-            subheadvec[2] = self.xtra1
-            dosubhead = True
-        if dosubhead:
-            report.text_cent(report.midpagew, report.h, 
-                       '\t'.join(subheadvec).strip(), report.fonts['subhead'])
+        if self.subheading is not None:
+            report.text_cent(report.midpagew, report.h, self.subheading,
+                           report.fonts['subhead'])
             report.h += report.line_height
-
         hof = report.h
         if len(self.lines) > 0:
             for i in self.lines:
@@ -704,9 +854,9 @@ class sprintfinal(object):
                 report.drawline(h12, ht, h12, hb)
                 report.drawline(h23, ht, h23, hb)
 
-                report.rms_rider(i[1], report.body_left+hw, hof)
+                report.sprint_rider(i[1], report.body_left+hw, hof)
                 hof += report.line_height
-                report.rms_rider(i[2], report.body_left+hw, hof)
+                report.sprint_rider(i[2], report.body_left+hw, hof)
                 hof += report.line_height
         report.h = hof
         report.c.restore()
@@ -722,8 +872,8 @@ class sprintfinal(object):
 class rttstartlist(object):
     def __init__(self):
         self.heading = None
+        self.subheading = None
         self.colheader = None
-        self.logo = None
         self.lines = []
         self.lcount = 0
         self.h = None
@@ -735,22 +885,24 @@ class rttstartlist(object):
             if self.colheader:	# colheader is written out with body
                 cvec.append(self.colheader)
             self.h = report.column_height(cvec)
-            if self.heading or self.logo:
+            if self.heading:
                 self.h += report.section_height
+            if self.subheading:
+                self.h += report.line_height
             self.lcount = len(self.lines)
         return self.h
 
     def truncate(self, remainder, report):
         """Return a copy of the section up to page break."""
 
-        # Special case 1: Entire section will fit on page, even if squeezed
+        # Special case 1: Entire section will fit on page
         if self.get_h(report) <= (remainder + report.page_overflow):
             return (self, None)
 
         # Special case 2: Not enough space for minimum content
         chk = rttstartlist()
         chk.heading = self.heading
-        chk.logo = self.logo
+        chk.subheading = self.subheading
         chk.colheader = self.colheader
         if len(self.lines) == 3: # special case, keep 'threes' together
             chk.lines = self.lines[0:]
@@ -760,13 +912,15 @@ class rttstartlist(object):
             # move entire section onto next page
             return (pagebreak(), self)
 
-        # Standard case - section must be broken
+        # Standard case - section crosses page break, determines
+        # ret: content on current page
+        # rem: content on subsequent pages
         ret = rttstartlist()
         rem = rttstartlist()
         ret.heading = self.heading
-        ret.logo = self.logo
+        ret.subheading = self.subheading
         rem.heading = self.heading
-        rem.logo = self.logo
+        rem.subheading = self.subheading
         if rem.heading is not None:
             if rem.heading.rfind('(continued)') < 0:
                 rem.heading += ' (continued)'
@@ -777,7 +931,7 @@ class rttstartlist(object):
         if seclines > 0:
             ret.lines.append(self.lines[0])
             count = 1 # case: 3 lines broken on first line
-        while count < seclines:
+        while count < seclines: # case: push min two lines over break
             if ret.get_h(report) > remainder:
                 # pop last line onto rem and break
                 rem.lines.append(ret.lines.pop(-1))
@@ -798,15 +952,15 @@ class rttstartlist(object):
             report.text_cent(report.midpagew, report.h, self.heading,
                            report.fonts['section'])
             report.h += report.section_height
-        if self.logo is not None:
-            ### make image elem and then draw it
-            pass
+        if self.subheading is not None:
+            report.text_cent(report.midpagew, report.h, self.subheading,
+                           report.fonts['subhead'])
+            report.h += report.line_height
         if len(self.lines) > 0:
             rows = []
             if self.colheader:
                 rows.append(self.colheader)
             rows.extend(self.lines)
-            # just hard-code cols for now, later do a colspec?
             report.output_column(rows, 0, 'r', report.col_oft_name)
             report.output_column(rows, 4, 'l', report.col_oft_name + mm2pt(2))
             report.output_column(rows, 1, 'r', report.col_oft_name + mm2pt(16))
@@ -820,9 +974,14 @@ class rttstartlist(object):
     def draw_xls(self, report, worksheet):
         """Output program element to excel worksheet."""
         row = report.h
-        if self.heading is not None:
+        if self.heading:
             worksheet.write(row, 2, self.heading.strip(), XS_TITLE)
+            row += 1
+        if self.subheading:
+            worksheet.write(row, 2, self.subheading.strip(), XS_SUBTITLE)
             row += 2
+        else:
+            row += 1
         if len(self.lines) > 0:
             rows = []
             if self.colheader:
@@ -848,8 +1007,10 @@ class rttstartlist(object):
 
     def draw_text(self, report, f):
         """Write out a section in markdown."""
-        if self.heading is not None:
+        if self.heading:
             f.write('### ' + self.heading.strip() + '\n\n')
+        if self.subheading:
+            f.write('#### ' + self.subheading.strip() + '\n\n')
         if len(self.lines) > 0:
             rows = []
             if self.colheader:
@@ -865,6 +1026,7 @@ class rttstartlist(object):
             f.write(htlib.table(trows, {'border':'0',
                                  'cellspacing':'2', 'cellpadding':'2'}))
             f.write('\n\n')
+        return None
 
 class bullet_text(object):
     """List of bullet items, each one a non-breaking pango para."""
@@ -980,7 +1142,32 @@ class bullet_text(object):
 
     def draw_xls(self, report, worksheet):
         """Output program element to excel worksheet."""
-        return None	# TODO
+        row = report.h
+        if self.heading:
+            worksheet.write(row, 2, self.heading.strip(), XS_TITLE)
+            row += 1
+        if self.subheading:
+            worksheet.write(row, 2, self.subheading.strip(), XS_SUBTITLE)
+            row += 2
+        else:
+            row += 1
+        if len(self.lines) > 0:
+            for l in self.lines:
+                oft = 0
+                bstr = self.bullet
+                if l[0]:
+                    bstr = l[0]
+                worksheet.write(row, 1, bstr, XS_LEFT)	# always one bullet
+                istr = ''
+                if l[1]: 
+                    istr = l[1]
+                for line in istr.split('\n'):
+                    worksheet.write(row + oft, 2, line, XS_LEFT)
+                    oft += 1
+                row += max(oft, 1)
+            row += 1
+        report.h = row
+        return None
 
     def draw_text(self, report, f):
         """Write out a section in markdown."""
@@ -1099,20 +1286,40 @@ class preformat_text(object):
 
     def draw_xls(self, report, worksheet):
         """Output program element to excel worksheet."""
-        return None	# TODO
+        row = report.h
+        if self.heading:
+            worksheet.write(row, 2, self.heading.strip(), XS_TITLE)
+            row += 1
+        if self.subheading:
+            worksheet.write(row, 2, self.subheading.strip(), XS_SUBTITLE)
+            row += 2
+        else:
+            row += 1
+        if len(self.lines) > 0:
+            if self.colheader:
+                worksheet.write(row, 2, self.colheader, XS_MONOSPACE)
+                row += 1
+            for l in self.lines:
+                worksheet.write(row, 2, l.rstrip(), XS_MONOSPACE)
+                row += 1
+            row += 1
+        report.h = row
+        return None
 
     def draw_text(self, report, f):
         """Write out a section in markdown."""
         if self.heading is not None:
             f.write('### ' + self.heading.strip() + '\n\n')
-        if self.subheading is not None:
-            f.write('#### ' + self.subheading.strip() + '\n\n')
+        if self.subheading:
+            f.write(htlib.p(htlib.escapetext(self.subheading.strip()), 
+                            {'class':'lead'}) + '\n\n')
         if len(self.lines) > 0:
+            ptxt = ''
             if self.colheader:
-                f.write('\t' + self.colheader.rstrip() + '\n')
+                ptxt += htlib.escapetext( self.colheader.rstrip()) + '\n'
             for row in self.lines:
-                f.write('\t' + row.rstrip() + '\n')
-            f.write('\n\n')
+                ptxt += htlib.escapetext(row.rstrip() + '\n')
+            f.write(htlib.pre(ptxt) + '\n\n')
 
 class section(object):
     def __init__(self):
@@ -1270,26 +1477,27 @@ class section(object):
         if self.heading:
             f.write('### ' + self.heading.strip() + '\n\n')
         if self.subheading:
-            f.write('#### ' + self.subheading.strip() + '\n\n')
+            f.write(htlib.p(htlib.escapetext(self.subheading.strip()), 
+                            {'class':'lead'}) + '\n\n')
+
         if len(self.lines) > 0:
-            rows = []
+            hdr = ''
             if self.colheader:
-                rows.append(self.colheader)
+                hdr = htlib.thead(vec2htmlhead(self.colheader))
+            rows = []
             for r in self.lines:
                 nv = r[0:6]
                 if len(nv) == 2:
                     nv = [nv[0], None, nv[1]]
                 rows.append(nv)
             if self.units:
-                if self.colheader:
-                    rows[1].append(self.units)
-                else:
-                    rows[0].append(self.units)
+                rows[0].append(self.units)
             trows = []
             for l in rows:
                 trows.append(vec2htmlrow(l))
-            f.write(htlib.table(trows, {'border':'0',
-                                 'cellspacing':'2', 'cellpadding':'2'}))
+            f.write(htlib.table([hdr, htlib.tbody(trows)],
+                  {'class':'table table-striped table-condensed',
+                   'style':'width: auto'}))
             f.write('\n\n')
         return None
 
@@ -1811,9 +2019,9 @@ class printrep(object):
         # Docstring?
         ws = wb.add_sheet(sheetname)
         # column widths
-        ws.col(0).width = int(5*256)
+        ws.col(0).width = int(7*256)
         ws.col(1).width = int(5*256)
-        ws.col(2).width = int(34*256)
+        ws.col(2).width = int(36*256)
         ws.col(3).width = int(11*256)
         ws.col(4).width = int(9*256)
         ws.col(5).width = int(7*256)
@@ -1852,20 +2060,26 @@ class printrep(object):
                 title += self.strings[s] + ' '
         cw.write(title.strip())
         cw.write('\n\n')
-        for s in ['datestr', 'docstr', 'diststr', 'commstr', 'orgstr']:
-            if s in self.strings and self.strings[s]:
-                cw.write('* ' + self.strings[s].strip() + '\n')
-
-        if len(linktypes) > 0:
-            linkmsg = '* Download as:'
-            for xtn in linktypes:
-                linkmsg += ' [' + xtn + '](' + linkbase + '.' + xtn + ')'
-            cw.write(linkmsg + '\n')
-        cw.write('\n')
 
         if self.provisional:
-            cw.write('*PROVISIONAL*\n\n')
+            cw.write('<span class="label label-warning pull-right">Provisional</span>\n\n')
 
+        metalist = []
+        for s in ['datestr', 'docstr', 'diststr', 'commstr', 'orgstr']:
+            if s in self.strings and self.strings[s]:
+                metalist.append((ICONMAP[s], htlib.escapetext(self.strings[s].strip())))
+        if len(linktypes) > 0:
+            linkmsg = 'Download as:'
+            for xtn in linktypes:
+                linkmsg += ' [' + htlib.a(xtn,
+                                   {'href':linkbase + '.' + xtn}) + ']'
+            metalist.append((ICONMAP['download'], linkmsg))
+        if len(metalist) > 0:
+            itemstr = ''
+            for li in metalist:
+                itemstr += htlib.li([htlib.i('',{'class':li[0]}),li[1]])
+            cw.write(htlib.div(htlib.ul(itemstr, {'class':'unstyled'}), 
+                              {'class':'well'}) + '\n\n')
         # output a jump trigger...
         cw.write('<!-- Jumper:jump -->\n\n')
 
@@ -2136,27 +2350,38 @@ class printrep(object):
                 self.drawline(w+mm2pt(8), baseline, w+mm2pt(58), baseline)
             self.drawline(w+mm2pt(59), baseline, w+mm2pt(75), baseline)
             
-    def ittt_heat(self, hvec, h, dual=False):
+    def ittt_heat(self, hvec, h, dual=False, showheat=True):
         """Output a single time trial heat."""
-        ## heat no provided, but ignored for pdf output?
-        if dual and hvec[0] and hvec[0] != '0':
+        if showheat:
             # allow for a heat holder but no text...
-            if hvec[0] != '-':
-                self.text_left(self.body_left, h, 'Heat ' + hvec[0],
+            if hvec[0] and hvec[0] != '-':
+                self.text_left(self.body_left, h, 'Heat ' + str(hvec[0]),
                                self.fonts['subhead'])
             h += self.line_height
-        # if riderno None - no rider at straight, else go for placeholder
         if len(hvec) > 3:	# got a front straight
             self.ittt_lane([hvec[1], hvec[2]], self.body_left, h)
-        if len(hvec) > 7: 
+        if len(hvec) > 7: 	# got a back straight
             if hvec[5] is not None:
                 self.text_cent(self.midpagew, h, 'v', self.fonts['subhead'])
             self.ittt_lane([hvec[5], hvec[6]], self.midpagew+mm2pt(5), h)
         elif dual:
-            # No Rider ??
-            #self.text_cent(self.midpagew, h, 'v', self.fonts['subhead'])
+            # No rider, but other heats are dual so add marker
             self.ittt_lane([None, None], self.midpagew+mm2pt(5), h)
 
+    def sprint_rider(self, rvec, w, h):
+        baseline = self.get_baseline(h)
+        # ignore rank in sprint round - defer to other markup
+        doline = True
+        if rvec[1]:	# rider no
+            self.text_right(w+mm2pt(5.0), h, rvec[1], self.fonts['body'])
+            doline = False
+        if rvec[2]:	# rider name
+            self.text_left(w+mm2pt(6.0), h, rvec[2], self.fonts['body'])
+            doline = False
+        if doline:
+            self.drawline(w+mm2pt(1.0), baseline, w+mm2pt(50), baseline)
+        # ignore cat/xtra in sprint rounds
+
     def rms_rider(self, rvec, w, h):
         baseline = self.get_baseline(h)
         if rvec[0] is not None:
