@@ -43,7 +43,6 @@ from metarace import eventdb
 from metarace import riderdb
 from metarace import strops
 from metarace import uiutil
-from metarace import thbc
 from metarace import printing
 
 # Model columns
@@ -639,6 +638,7 @@ class rms(object):
         """Return the judges (camera) report."""
         ret = []
         self.recalculate()	# fill places and bunch info
+        pthresh = self.meet.timer.photothresh()
         totcount = 0
         dnscount = 0
         dnfcount = 0
@@ -669,7 +669,7 @@ class rms(object):
                         # format 'elapsed' rftime
                         if r[COL_RFTIME] is not None:
                             if not pset and lrf is not None:
-                                if r[COL_RFTIME]-lrf < thbc.PHOTOTHRESH:
+                                if r[COL_RFTIME]-lrf < pthresh:
                                     comment = '[pf]__'
                                     prcom = sec.lines[-1][0]
                                     if prcom in ['___', '[pf]__']:
@@ -1095,60 +1095,6 @@ class rms(object):
                     penalty = r[COL_PENALTY]
                 yield [crank, bib, ft, bonus, penalty]
 
-    # main shared result export -> outputs text/csv lines for spreadsheet
-    # TEMP -> Dump rows in Lyndon's ASCC format :/
-    def result_export(self, f):
-        """Export result for use with other systems."""
-        self.recalculate()	# fix up ordering of rows
-        cr = csv.writer(f)
- 
-        if self.timerstat != 'idle':
-            first = True
-            ft = None
-            lt = None
-            cr.writerow(['', 'Place', 'Race No', 'Name', 'Team', 
-                             'Region', 'Gender', 'Division', 'Time'])
-            #cr.writerow(['place', 'no.', 'cat', 'time'])
-            for r in self.riders:
-                vbs = ''
-
-                namestr = '[missing]'
-                clubstr = '[none]'
-                dbr = self.meet.rdb.getrider(r[COL_BIB],self.series)
-                if dbr is not None:
-                    namestr = strops.fitname(
-                                self.meet.rdb.getvalue(dbr, riderdb.COL_FIRST),
-                                self.meet.rdb.getvalue(dbr, riderdb.COL_LAST),
-                                64).strip()
-                    if namestr == '':
-                        namestr = '[missing]'
-                    clubstr = self.meet.rdb.getvalue(dbr, riderdb.COL_CLUB)
-                    if clubstr == '':
-                        clubstr = '[none]'
-
-                if r[COL_INRACE]:
-                    bt = self.vbunch(r[COL_CBUNCH], r[COL_MBUNCH])
-                    ft = bt
-                    if r[COL_STOFT] is not None:
-                        ft = bt - r[COL_STOFT]
-                    if bt is not None:
-                        vbs = "'" + ft.rawtime(0)
-                        #vbs = "'" + ft.rawtime(0, zeros=True,
-                                               #hoursep='.', minsep='.')
-                        #vbs = "'" + ft.rawtime(2, zeros=True)
-                    cr.writerow(['', r[COL_PLACE], r[COL_BIB],
-                                 namestr, clubstr, '', '', r[COL_CAT], vbs])
-                else:
-                    comment = r[COL_COMMENT]
-                    if comment == '':
-                        comment = 'dnf'
-                    cr.writerow(['', comment, r[COL_BIB],
-                                 namestr, clubstr, '', '',
-                                 r[COL_CAT], comment])
-            # Terminator rows
-            cr.writerow([' ', '', '', '', '', '', '', '', ' '])
-            cr.writerow(['[eod]', '', '', '', '', '', '', '', '[eod]'])
-
     def clear_results(self):
         """Clear all data from event model."""
         self.resetplaces()
@@ -1575,7 +1521,7 @@ class rms(object):
         return False
 
     def timeout(self):
-        """Poll for rfids and update elapsed time."""
+        """Update elapsed time and recalculate if required."""
         if not self.winopen:
             return False
         #if self.finish is None and self.start is not None:
@@ -2344,4 +2290,3 @@ class rms(object):
             #self.view.connect('row-activated', self.treerow_selected)
             b.connect_signals(self)
             self.meet.timer.setcb(self.rfidtrig)
-            self.meet.edb.editevent(event, winopen=True)
