@@ -1,5 +1,4 @@
 ## TODO:	- merge autoclear announce into notepad
-##		- replace wheeltime with thbc
 ##		- unified announcer output
 ##		- multi-announce framework
 ##		- remote control via uscbsrv
@@ -51,6 +50,7 @@ from metarace import tod
 from metarace import eventdb
 from metarace import riderdb
 from metarace import thbc
+from metarace import wheeltime
 from metarace import timy
 from metarace import uscbsrv
 from metarace import unt4
@@ -58,6 +58,7 @@ from metarace import strops
 from metarace import loghandler
 from metarace import uiutil
 from metarace import printing
+from metarace import rsync
 
 LOGHANDLER_LEVEL = logging.DEBUG
 ROADRACE_TYPES = {'irtt':'Road Time Trial',
@@ -69,7 +70,9 @@ ROADRACE_TYPES = {'irtt':'Road Time Trial',
 CONFIGFILE = 'config.ini'
 LOGFILE = 'event.log'
 ROADMEET_ID = 'roadmeet_1.5'  # configuration versioning
-
+DEFAULT_RFID_HANDLER = 'thbc'
+RFID_HANDLERS = {'thbc':thbc.thbc,
+                 'wheeltime':wheeltime.wheeltime}
 # Bunches colourmap
 COLOURMAP=[['#ffa0a0','red',1.0,0.1,0.1],
            ['#a0ffa0','green',0.1,1.0,0.1],
@@ -81,6 +84,18 @@ COLOURMAP=[['#ffa0a0','red',1.0,0.1,0.1],
            ['#00ffc3','cyan',0.1,1.0,0.8]]
 COLOURMAPLEN=len(COLOURMAP)
 
+def rfid_device(devstr=''):
+    """Return a pair: (device, port) for the provided device string."""
+    (a, b, c) = devstr.partition(':')
+    devtype = DEFAULT_RFID_HANDLER
+    if b:
+        a = a.lower()
+        if a in RFID_HANDLERS:
+            devtype = a
+        a = c	# shift port into a
+    devport = a
+    return((devtype, devport))
+
 class fakemeet:
     """Road meet placeholder for external event manipulations."""
     def __init__(self, edb, rdb, path):
@@ -211,7 +226,7 @@ class roadmeet:
         mte.set_text(self.timer_port)
         mtb = b.get_object('timing_main_dfl')
         mtb.connect('clicked',
-                    lambda b: mte.set_text(thbc.DEFPORT))
+                    lambda b: mte.set_text('thbc:' + thbc.DEFPORT))
         ate = b.get_object('timing_alt_entry')
         ate.set_text(self.alttimer_port)
         atb = b.get_object('timing_alt_dfl')
@@ -236,10 +251,7 @@ class roadmeet:
                 self.uscbchan = nchan
                 self.scb.set_portstr(portstr=self.uscbport,
                                      channel=self.uscbchan)
-            nport = mte.get_text()
-            if nport != self.timer_port:
-                self.timer_port = nport
-                self.timer.setport(nport)
+            self.set_timer(mte.get_text())
             nport = ate.get_text()
             if nport != self.alttimer_port:
                 self.alttimer_port = nport
@@ -360,6 +372,9 @@ class roadmeet:
             elif self.etype == 'cross':
                 from metarace import cross
                 self.curevent = cross.cross(self, eventhdl, True)
+            elif self.etype == 'sportif':
+                from metarace import sportif
+                self.curevent = sportif.sportif(self, eventhdl, True)
             else:	# default is fall back to road mass start 'rms'
                 from metarace import rms
                 self.curevent = rms.rms(self, eventhdl, True)
@@ -581,23 +596,68 @@ class roadmeet:
                     cw.writerow(r)
             self.log.info('Exported startlist to ' + repr(rfilename))
     
-    ## TODO: After data structs and reporting in curevent determined,
-    ##       use a walk method to build html result files, and flag
-    ##       or spawn a ftp uploader... requires some sort of finish
-    ##       flag.
     def menu_data_results_cb(self, menuitem, data=None):
-        """Call into curevent to create a set of results."""
-        self.log.debug('Live result not yet implemented.')
-        return
-        #from metarace import htlib
-        #if self.curevent is not None:
-            #rfilename = os.path.join(self.configpath, 'results.html')
-            #with open(rfilename, 'wb') as f:
-                #result = ''
-                #if self.curevent is not None:
-                    #result = self.curevent.result_export('')
-                ###body.append(htlib.h1(racebname))
-                #f.write(htlib.html(htlib.title('Test Header'), result))
+        """Create live result report and export to MR"""
+        if self.curevent is None:
+            return
+
+        # 1: make report with meta from event and meet
+        tfile = os.path.join(self.configpath, 'template.ini')
+        if not os.path.exists(tfile):
+            tfile = None
+        rep = printing.printrep(template=tfile, path=self.configpath)
+        rep.strings['title'] = ''	# use the subtitle only
+        rep.strings['subtitle'] = self.title_str
+        rep.strings['datestr'] = strops.promptstr('Date:', self.date_str)
+        rep.strings['commstr'] = strops.promptstr('Chief Commissaire:',
+                                                  self.commissaire_str)
+        rep.strings['orgstr'] = strops.promptstr('Organiser:',
+                                                  self.organiser_str)
+        rep.strings['diststr'] = 'Location: Marysville, Victoria'
+        rep.strings['docstr'] = ''
+        #if self.distance:
+            #rep.strings['diststr'] = strops.promptstr('Distance:',
+                                                  #str(self.distance) + ' km')
+        #else:
+            #rep.strings['diststr'] = ''
+
+
+        # 2: ensure expot path valid
+        exportpath = os.path.join(self.configpath, 'export')
+        if not os.path.exists(exportpath):
+            os.mkdir(exportpath)
+ 
+        # 3: set provisional status
+        if self.curevent.timerstat != 'finished':
+            rep.set_provisional(True)
+
+        # 4: call into curevent to get result sections
+        for sec in self.curevent.result_report():
+            rep.add_section(sec)
+
+        # 5: write out files
+        filebase = os.path.basename(self.configpath)
+        self.log.info('was: ' + repr(self.configpath) + ', is: ' + 
+                       repr(filebase))
+        if filebase == '':
+            filebase = 'result'
+            self.log.warn('Using default flename for export: result')
+
+        ofile = os.path.join(exportpath, filebase+'.pdf')
+        with open(ofile, 'wb') as f:
+            rep.output_pdf(f)
+        ofile = os.path.join(exportpath, filebase+'.xls')
+        with open(ofile, 'wb') as f:
+            rep.output_xls(f)
+        lb = os.path.join('/site', self.mirrorpath, filebase)
+        lt = ['pdf', 'xls']
+        ofile = os.path.join(exportpath, filebase+'.txt')
+        with open(ofile, 'wb') as f:
+            rep.output_text(f, linkbase=lb, linktypes=lt)
+
+        # 6: if not in progress, create and trigger rsync proc
+        glib.idle_add(self.mirror_start)
+        self.log.info('Race info export.')
 
     ## Directory utilities
     def event_configfile(self, evno):
@@ -658,7 +718,8 @@ class roadmeet:
 
     def menu_timing_reconnect_activate_cb(self, menuitem, data=None):
         """Reconnect timer and re-start."""
-        self.timer.setport(self.timer_port)
+        (cdev, cport) = rfid_device(self.timer_port)
+        self.timer.setport(cport)	# force reconnect
         self.timer.stop_session()
         self.timer.sane()
         self.timer.start_session()
@@ -716,17 +777,10 @@ class roadmeet:
             if self.curevent is not None:
                 self.curevent.timeout()
 
-            # lastly display RFU status button
-            ## TODO: replace this code somehow with status callback
-            #nstat = self.rfu.connected()
-            #if nstat != self.rfustat:
-                #if nstat:
-                    #self.menu_rfustat_img.set_from_stock(
-                           #gtk.STOCK_CONNECT, gtk.ICON_SIZE_LARGE_TOOLBAR)
-                #else:
-                    #self.menu_rfustat_img.set_from_stock(
-                           #gtk.STOCK_DISCONNECT, gtk.ICON_SIZE_LARGE_TOOLBAR)
-                #self.rfustat = nstat
+            # check for completion in the rsync module
+            if self.mirror is not None:
+                if not self.mirror.is_alive():
+                    self.mirror = None
         else:
             return False
         return True
@@ -786,13 +840,25 @@ class roadmeet:
     def shutdown(self, msg=''):
         """Cleanly shutdown threads and close application."""
         if self.curevent is not None:
-            self.curevent.destroy()
+            self.curevent.destroy()	# necessary here?
+
+        self.started = False
+
         self.window.hide()
-        self.scb.clrall()
         self.timer.exit(msg)
         self.alttimer.exit(msg)
         self.scb.exit(msg)
-        self.started = False
+        print ('Waiting for workers to exit...')
+        if self.mirror is not None:
+            print('\tlive result mirror.')
+            self.mirror.join()
+            self.mirror = None
+        print('\tmain timer.')
+        self.timer.join()
+        print('\talt timer.')
+        self.alttimer.join()
+        print('\tannouncer.')
+        self.scb.join()
 
     def start(self):
         """Start the timer and rfu threads."""
@@ -826,6 +892,7 @@ class roadmeet:
         cw.set('meet', 'distance', str(self.distance))
         cw.set('meet', 'docindex', str(self.docindex))
         cw.set('meet', 'loglevel', str(self.loglevel))
+        cw.set('meet', 'mirrorpath', self.mirrorpath)
 
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         self.log.debug('Saving meet config to ' + repr(cwfilename))
@@ -839,6 +906,26 @@ class roadmeet:
         if self.pageset is not None:
             self.pageset.to_file(os.path.join(self.configpath, 'page.prf'))
 
+    def set_timer(self, newdevice=''):
+        """Re-set the main timer devices as specified."""
+        # Step 1: is a change required?
+        if newdevice != self.timer_port:
+            (dev, nport) = rfid_device(newdevice)
+            if type(self.timer) is RFID_HANDLERS[dev]:
+                self.log.debug('timer thread same type - no change required.')
+                self.timer.setport(nport)
+            else:	# need to 'switch' to new handler
+                wasalive = self.timer.is_alive()
+                self.timer.exit('Switching devices.')
+                self.timer = None	# unref
+                self.timer = RFID_HANDLERS[dev](port=nport)
+                if wasalive:
+                    self.timer.start()
+            # save change
+            self.timer_port = newdevice 
+        else:
+            self.log.debug('set_timer - No change required.')
+
     def loadconfig(self):
         """Load meet config from disk."""
         cr = ConfigParser.ConfigParser({
@@ -855,6 +942,7 @@ class roadmeet:
                'uscbport':'',
                'uscbchan':'#announce',
                'uscbopt':'No',
+               'mirrorpath':'',
                'loglevel':str(logging.INFO),
                'id':''
         })
@@ -888,11 +976,7 @@ class roadmeet:
                              channel=self.uscbchan)
 
         # set timer port (decoder)
-        nport = cr.get('meet', 'timer')
-        if nport != self.timer_port:
-            self.timer_port = nport
-            self.timer.setport(nport)
-            self.timer.sane() # sane prod here is probably good idea
+        self.set_timer(cr.get('meet', 'timer'))
 
         # set alt timer port (timy)
         nport = cr.get('meet', 'alttimer')
@@ -910,6 +994,7 @@ class roadmeet:
         self.distance = strops.confopt_float(cr.get('meet', 'distance'))
         self.docindex = strops.confopt_posint(cr.get('meet', 'docindex'), 0)
         self.bibs_in_results = strops.confopt_bool(cr.get('meet', 'resultnos'))
+        self.mirrorpath = cr.get('meet', 'mirrorpath')
 
         # configurable log level on UI (does not change log file)
         self.loglevel = strops.confopt_posint(cr.get('meet', 'loglevel'),
@@ -1026,14 +1111,23 @@ class roadmeet:
             else:
                 # Informative non-timeline record
                 nr=[rvec[0],rvec[1],rvec[2],rvec[3],
-                        '', '', '#fefefe',None]
+                        rvec[4], '', '#fefefe',None]
             self.announce_model.append(nr)
         else:
-            self.log.info('Ignoed unknown rider announce vector')
+            self.log.info('Ignored unknown rider announce vector')
         self.scb.add_rider(rvec)
 
         return False	# so can be idle-added
 
+    def mirror_start(self):
+        """Create a new rsync mirror thread unless in progress."""
+        if self.mirrorpath and self.mirror is None:
+            self.mirror = rsync.mirror(
+                localpath = os.path.join(self.configpath,'export',''),
+                remotepath = self.mirrorpath)
+            self.mirror.start()
+        return False    # for idle_add
+
     def __init__(self, configpath=None, etype=None):
         """Meet constructor."""
         # logger and log handler
@@ -1072,13 +1166,15 @@ class roadmeet:
         self.pageset.set_right_margin(0, gtk.UNIT_POINTS)
 
         # hardware connections
-        self.timer = thbc.thbc()
+        self.timer = thbc.thbc()	# default is Tag, may change later
         self.timer_port = ''
         self.alttimer = timy.timy()
         self.alttimer_port = ''
         self.uscbport = ''
         self.uscbchan = '#announce'
         self.scb = uscbsrv.uscbsrv()
+        self.mirrorpath = ''    # default rsync mirror path
+        self.mirror = None 
 
         b = gtk.Builder()
         b.add_from_file(os.path.join(metarace.UI_PATH, 'roadmeet.ui'))
