@@ -18,9 +18,8 @@
 """Mass participartion 'sportif' ride handler.
 
 This module provides a class 'sportif' which implements the sportif
-ride handler. Written for the 2010 'Ride the Worlds' sportif, this
-is a rather clumsy event handler but should serve as a starting
-point for a custom event handler as it implements only the required
+ride handler. Written for the 2010 'Ride the Worlds' sportif, and modified
+for the Netti Challenge series, it implements only the required
 methods to fit into the roadmeet framework.
 
 """
@@ -34,11 +33,9 @@ methods to fit into the roadmeet framework.
 import gtk
 import glib
 import gobject
-import pango
 import os
 import logging
 import csv
-import decimal
 import ConfigParser
 
 import metarace
@@ -46,7 +43,7 @@ from metarace import tod
 from metarace import eventdb
 from metarace import riderdb
 from metarace import strops
-from metarace import printops
+from metarace import printing
 from metarace import uiutil
 
 # Model columns
@@ -56,11 +53,13 @@ COL_BIB = 0
 COL_NAMESTR = 1
 COL_CAT = 2
 COL_COMMENT = 3
+COL_SORTKEY = 4
+COL_TEAM = 5
 
 # timing infos
-COL_START = 4		# one-off start time or None for mass start
-COL_RFTIME = 5		# one-off finish time (if relevant) by rfid
-COL_RFSEEN = 6		# list of tods this rider 'seen' by rfid
+COL_START = 6		# one-off start time or None for mass start
+COL_RFTIME = 7		# one-off finish time (if relevant) by rfid
+COL_RFSEEN = 8		# list of tods this rider 'seen' by rfid
 
 # rider commands
 RIDER_COMMANDS_ORD = ['add', 'del', 'que', 'com']
@@ -79,15 +78,19 @@ key_raceover = 'F10'
 key_abort = 'F5'
 
 # config version string
-EVENT_ID = 'sportif-1.1'
+EVENT_ID = 'sportif-1.4'
 
-def sort_bib(x, y):
-    """Rider bib sorter."""
-    if x.isdigit():
-        x = int(x)
-    if y.isdigit():
-        y = int(y)
-    return cmp(x, y)
+def key_bib(x):
+    """Sort on bib field of aux row."""
+    return strops.riderno_key(x[1])
+
+def key_name(x):
+    """Sort on sort order key."""
+    return x[1]
+
+def key_team_res(x):
+    """Sort on team aggregate time."""
+    return int(x[4].timeval)
 
 class sportif(object):
     """Sportif ride handler."""
@@ -100,6 +103,9 @@ class sportif(object):
                                         'lstart':'',
                                         'id':EVENT_ID,
                                         'finish':'',
+                                        'categories':'',
+                                        'minimums':'',
+                                        'maximums':'',
                                         'finished':'No',
                                         'startlist':''})
         cr.add_section('event')
@@ -108,9 +114,37 @@ class sportif(object):
             self.log.debug('Attempting to read config from path='
                             + repr(self.configpath))
             cr.read(self.configpath)
+
+        # load _result_ categories
+        catlist = cr.get('event', 'categories')
+        self.cats = []
+        for cat in catlist.split(','):
+            if cat != '':
+                cat = cat.strip().upper()
+                self.cats.append(cat)
+        self.cats.append('')    # always include one empty cat
+        minlist = []
+        for mtm in cr.get('event', 'minimums').split(','):
+            minlist.append(tod.str2tod(mtm))
+        while len(minlist) < len(self.cats):
+            minlist.append(None)	# extend as necessary
+
+        # and repead for maxlist
+        maxlist = []
+        for mtm in cr.get('event', 'maximums').split(','):
+            maxlist.append(tod.str2tod(mtm))
+        while len(maxlist) < len(self.cats):
+            maxlist.append(None)	# extend as necessary
+
+        # then populate maps	HACK	should be stored this way
+        self.minmap = {}
+        self.maxmap = {}
+        for j in range(0, len(self.cats)):
+            self.minmap[self.cats[j]] = minlist[j]
+            self.maxmap[self.cats[j]] = maxlist[j]
+
         starters = cr.get('event', 'startlist').split()
         # for a sportif - always sort by bib
-        starters.sort(cmp=sort_bib)
         for r in starters:
             self.addrider(r)
             if cr.has_option('riders', r):
@@ -143,7 +177,6 @@ class sportif(object):
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
 
-## TODO: This is dodgey, req a better sorted method to include intermeds
     def get_ridercmdorder(self):
         return RIDER_COMMANDS_ORD
 
@@ -177,6 +210,25 @@ class sportif(object):
         else:
             cw.set('event', 'finished', 'No')
         cw.set('event', 'startlist', self.get_startlist())    
+        catlist = []
+        minlist = []
+        maxlist = []
+        for cat in self.cats:
+            if cat:
+                catlist.append(cat)
+                mtm = self.minmap[cat]
+                if mtm is not None:
+                    minlist.append(mtm.rawtime())
+                else:
+                    minlist.append('')
+                mtm = self.maxmap[cat]
+                if mtm is not None:
+                    maxlist.append(mtm.rawtime())
+                else:
+                    maxlist.append('')
+        cw.set('event', 'categories', ','.join(catlist))
+        cw.set('event', 'minimums', ','.join(minlist))
+        cw.set('event', 'maximums', ','.join(maxlist))
 
         cw.add_section('riders')
         for r in self.riders:
@@ -224,80 +276,144 @@ class sportif(object):
         """Extract results in flat mode (not yet implemented)."""
         return []
 
-    def startlist_header(self):
-        """Return a startlist header."""
-        return('no.   rider')
+    def reorder_startlist(self):
+        """Reorder riders for a startlist."""
+        aux = []
+        cnt = 0
+        for r in self.riders:
+            aux.append([cnt, r[COL_BIB]])
+            cnt += 1
+        if len(aux) > 1:
+            aux.sort(key=key_bib)
+            self.riders.reorder([a[0] for a in aux])
+        return cnt
 
     def startlist_report(self):
-        """Return a startlist."""
+        """Return a startlist report."""
         ret = []
+        sec = printing.section()
+        sec.heading = 'Startlist'
+        cnt = self.reorder_startlist()
         for r in self.riders:
-            ret.append(r[riderdb.COL_BIB].rjust(4) + '  '
-                                 + strops.truncpad(r[COL_NAMESTR], 48)
-                                 + str(r[COL_CAT]).rjust(10))
+            sec.lines.append([None, r[COL_BIB], r[COL_NAMESTR], r[COL_CAT]])
+        ret.append(sec)
+        if cnt > 1:
+            sec = printing.section()
+            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
+            ret.append(sec)
         return ret
 
-    def result_header(self):
-        """Return the result header."""
-        return('\
-no.   rider                                   time  avg.')
+    def reorder_byname(self):
+        """Reorder riders for a result."""
+        aux = []
+        cnt = 0
+        for r in self.riders:
+            aux.append([cnt, r[COL_SORTKEY]])
+            cnt += 1
+        if len(aux) > 1:
+            aux.sort(key=key_name)
+            self.riders.reorder([a[0] for a in aux])
+        return cnt
 
     def result_report(self):
         """Return a result report."""
         ret =  []
+        tmap = {}	# teams result map
 
-        # set the start time
+        # set the 'start' time
         st = tod.tod(0)
         if self.start is not None:
             st = self.start
 
-        # resort -> ??
+        sec = printing.section()
+        sec.heading = 'Individuals'
+
+        self.reorder_byname()
 
-        # scan registered riders
         for r in self.riders:
-            ret.append(self.fmt_rider_result(r, st))
-        return ret
+            et = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
+            cat = self.ridercat(r[COL_CAT])
+            if et is not None and cat:
+                if self.minmap[cat] is not None and et < self.minmap[cat]:
+                    self.log.warn('Rider time less than minimum: '
+                                     + repr(r[COL_BIB]))
+                    et = None	
+                elif self.maxmap[cat] is not None and et > self.maxmap[cat]:
+                    self.log.warn('Rider time greater than maximum: '
+                                     + repr(r[COL_BIB]))
+                    et = None	
+            estr = ''
+            if et is not None:
+                if r[COL_TEAM]:
+                    if r[COL_TEAM] not in tmap:
+                        tmap[r[COL_TEAM]] = []	# vec of elapsed times
+                    tmap[r[COL_TEAM]].append(et)
+                estr = et.rawtime(0)
+            elif r[COL_START] is not None:
+                estr = '[on course]'
+            else:
+                estr = '[not started]'
 
-    def camera_header(self):
-        """Return the judges report header."""
-        return ''
+            sec.lines.append([None, r[COL_BIB], r[COL_NAMESTR], r[COL_CAT],
+                                    estr, None])
+        ret.append(sec)
+
+        sec = printing.section()
+        sec.heading = 'Teams'
+        tres = []
+        for team in tmap:
+            rvec = tmap[team]
+            if len(rvec) > 1:
+                rvec.sort()
+                aggtm = rvec[0].truncate(0) + rvec[1].truncate(0)
+                tres.append([None, None, team, None, aggtm, None])
+        if len(tres) > 0:
+            tres.sort(key=key_team_res)
+            for res in tres:
+                res[4] = res[4].rawtime(0)
+                sec.lines.append(res)
+        ret.append(sec)
+        return ret
 
     def camera_report(self):
         """Return a judges (camera) report."""
-        self.log.error('Judges report not implemented for sportif rides.')
-        return ['     -- No Judges Report for Sportif Ride --']
+        self.log.error('Judges report not available for sportif rides.')
+        return None
+
+    def points_report(self):
+        """Return a points tally report."""
+        self.log.error('Points tally report not available for sportif rides.')
+        return None
 
     def stat_but_clicked(self):
         """Deal with a status button click in the main container."""
         self.log.debug('Stat button clicked.')
 
     def fmt_rider_result(self, r, st):
-        """Return a result string for the provided rider reference."""
-# 0123456789012345678901
-# [not started]         |
-# [on course]           |
-# 12h13:12.23  nn.n km/h|
+        """Return a string for the provided rider reference."""
+# 0123456789012		# used in the scratch view - leave for now
+# [not started]|
+# [on course]|
+# 12h13:12.2|
         fstr = ''
         et = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
         if et is not None:
-            fstr = et.rawtime(2).rjust(11)
-            di = self.meet.get_distance()
-            if di:
-                fstr += '  {0:.1f} km/h'.format(3600 * float(di) / float(et.timeval))
+            fstr = et.rawtime(1)
+            #di = self.meet.get_distance()
+            #if di:
+                #fstr += '  {0:.1f} km/h'.format(3600 * float(di) / float(et.timeval))
+            # distance calcs require some thought and cat lookups
         elif r[COL_START] is not None:
             fstr = '[on course]'
         else:
             fstr = '[not started]'
-        ret = ' '.join([r[COL_BIB].rjust(4), '',
-                        strops.truncpad(r[COL_NAMESTR], 32),
-                        fstr.ljust(22)])
-        if False:			# ignore intermeds
-        #if len(r[COL_RFSEEN]) > 3:
-            lt = st
-            for split in r[COL_RFSEEN]:
-                ret += ' ' + (split - lt).rawtime(0).rjust(7)
-                lt = split
-        return(ret)
+        
+        #if len(r[COL_RFSEEN]) > 3:	# ignore intermeds
+            #lt = st
+            #for split in r[COL_RFSEEN]:
+                #ret += ' ' + (split - lt).rawtime(0).rjust(7)
+                #lt = split
+        return fstr
 
     def query_rider(self, bib=None):
         """List info on selected rider in the scratchpad."""
@@ -310,7 +426,10 @@ no.   rider                                   time  avg.')
         # get the rider
         r = self.getrider(bib)
         if r is not None:
-            self.meet.scratch_log(self.fmt_rider_result(r, st))
+            fstr = self.fmt_rider_result(r, st)
+            self.log.info('Rider ' + repr(bib) + ' : ' + repr(fstr))
+            self.meet.announce_rider(['',bib,r[COL_NAMESTR],
+                                     r[COL_CAT],fstr])
         else:
             self.log.info('Rider = ' + repr(bib) + ' not in startlist.')
 
@@ -320,6 +439,11 @@ no.   rider                                   time  avg.')
         """Append a race comment."""
         self.log.info('Add comment: ' + repr(comment))
 
+    def ctrl_change(self, acode='', entry=None):
+        """Notify change in action combo."""
+        if entry is not None:
+            entry.set_text('')
+
     def race_ctrl(self, acode='', rlist=''):
         """Apply the selected action to the provided bib list."""
         if acode == 'del':
@@ -346,40 +470,46 @@ no.   rider                                   time  avg.')
             self.log.error('Ignoring invalid action.')
         return False
 
-    def result_export(self, f):
-        """Export result for use with other systems."""
-        cr = csv.writer(f)
- 
-        cr.writerow(['no', 'rider', 'email', 'laps', 
-                     'Split 1', 'Lap 1', 'Split 2', 'Lap 2',
-                     'Split 3', 'Lap 3', 'Split 4', 'Lap 4'])
-
-        # set the start time
-        st = tod.tod(0)
-        if self.start is not None:
-            st = self.start
-
-        # scan start list
+    def startlist_gen(self, cat=''):
+        """Generator function to export a startlist."""
+        mcat = self.ridercat(cat)
+        self.reorder_startlist()
         for r in self.riders:
-            res = [r[COL_BIB], r[COL_NAMESTR], r[COL_COMMENT],
-                   str(len(r[COL_RFSEEN]))]
+            if mcat == '' or mcat == self.ridercat(r[COL_CAT]):
+                start = ''
+                if r[COL_START] is not None and r[COL_START] != tod.ZERO:
+                    start = r[COL_START].rawtime(1)
+                bib = r[COL_BIB]
+                series = self.series
+                name = r[COL_NAMESTR]
+                cat = r[COL_CAT]
+                firstxtra = ''
+                lastxtra = ''
+                clubxtra = r[COL_TEAM]
+                dbr = self.meet.rdb.getrider(r[COL_BIB],self.series)
+                if dbr is not None:
+                    firstxtra = self.meet.rdb.getvalue(dbr,
+                                         riderdb.COL_FIRST).capitalize()
+                    lastxtra = self.meet.rdb.getvalue(dbr,
+                                         riderdb.COL_LAST).upper()
+                yield [start, bib, series, name, cat,
+                       firstxtra, lastxtra, clubxtra]
 
-            # scan lap splits
-            lt = st
-            for split in r[COL_RFSEEN]:
-                laptime = split - lt
-                lapstr = laptime.rawtime(0)
-                splitstr = split.rawtime(0)
-                if laptime < self.minlap:
-                    self.log.warn('Suspect lap for rider ' 
-                                  + repr(r[COL_BIB]) + ':'
-                                  + lapstr)
-                res.append(splitstr)
-                res.append(lapstr)
-                lt = split
-
-            # emit row
-            cr.writerow(res)
+    def result_gen(self, cat=''):
+        """Generator function to export a final result."""
+        self.recalculate()
+        self.reorder_startlist()
+        mcat = self.ridercat(cat)
+        rcount = 0
+        lrank = None
+        lcrank = None
+        for r in self.riders:
+            if mcat == '' or mcat == self.ridercat(r[COL_CAT]):
+                rcount += 1
+                bib = r[COL_BIB]
+                crank = None
+                ft = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
+                yield [crank, bib, ft, None, None]
 
     def clear_results(self):
         """Clear all data from event model."""
@@ -412,14 +542,17 @@ no.   rider                                   time  avg.')
     def addrider(self, bib=''):
         """Add specified rider to race model."""
         if bib == '' or self.getrider(bib) is None:
-            nr = [bib, '', '', '', None, None, []]
+            nr = [bib, '', '', '', '', '', None, None, []]
             dbr = self.meet.rdb.getrider(bib, self.series)
             if dbr is not None:
-                nr[COL_NAMESTR] = strops.listname(
-                      self.meet.rdb.getvalue(dbr, riderdb.COL_FIRST),
-                      self.meet.rdb.getvalue(dbr, riderdb.COL_LAST),
-                      self.meet.rdb.getvalue(dbr, riderdb.COL_CLUB))
+                first = self.meet.rdb.getvalue(dbr, riderdb.COL_FIRST)
+                last = self.meet.rdb.getvalue(dbr, riderdb.COL_LAST)
+                club = self.meet.rdb.getvalue(dbr, riderdb.COL_CLUB)
+
+                nr[COL_NAMESTR] = strops.listname(first, last, club)
                 nr[COL_CAT] = self.meet.rdb.getvalue(dbr, riderdb.COL_CAT)
+                nr[COL_SORTKEY] = last.lower().ljust(30) + first.lower()
+                nr[COL_TEAM] = club
             return self.riders.append(nr)
         else:
             return None
@@ -430,7 +563,6 @@ no.   rider                                   time  avg.')
         self.set_start()
         self.clear_results()
         self.timerstat = 'idle'
-        self.meet.rfu.dearm()
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle')
         self.meet.stat_but.set_sensitive(True)
         self.set_elapsed()
@@ -439,13 +571,11 @@ no.   rider                                   time  avg.')
         """Process an armstart request."""
         if self.timerstat == 'idle':
             self.timerstat = 'armstart'
-            self.meet.rfu.arm()
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_armstart,
                                     'Arm Start')
         elif self.timerstat == 'armstart':
             self.timerstat = 'idle'
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle') 
-            self.meet.rfu.dearm()	# superfluous?
         elif self.timerstat == 'running':
             # Possible extra state transition here in response to F5
             pass
@@ -456,7 +586,6 @@ no.   rider                                   time  avg.')
             self.timerstat = 'armfinish'
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_armfin, 'Arm Finish')
             self.meet.stat_but.set_sensitive(True)
-            self.meet.rfu.arm()
         elif self.timerstat == 'armfinish':
             self.timerstat = 'running'
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
@@ -480,7 +609,8 @@ no.   rider                                   time  avg.')
                     self.set_finished()
                     return True
                 elif key == key_clearscratch:
-                    self.meet.scratch_clear()
+                    self.meet.announce_clear()
+                    self.meet.announce_title(self.meet.title_str)
                     return True
         return False
 
@@ -496,13 +626,14 @@ no.   rider                                   time  avg.')
         """Process a 'start' trigger signal."""
         if self.timerstat == 'armstart':
             self.set_start(e, tod.tod('now'))
+        return False
 
     def rfidtrig(self, e):
         """Process rfid event."""
-        #if e.refid == 'trig':
-            #self.starttrig(e)
-            #return
+        if e.refid == '':	# assume trigger
+            return self.starttrig(e)
 
+        # else assume a passing
         r = self.meet.rdb.getrefid(e.refid)
         if r is None:
             r = self.meet.rdb.addempty(e.refid, self.series)
@@ -515,19 +646,18 @@ no.   rider                                   time  avg.')
             return
 
         lr = self.getrider(bib)
-        if lr is None:
+        if lr is None:	# sportif uses relaxed 'club mode'
             self.addrider(bib)
             lr = self.getrider(bib)
             self.log.info('Added non starter: ' + bib
                           + ' @ ' + e.rawtime(1))
 
         # at this point should always have a valid rider vector
-        assert(lr is not None)
-
         if self.timerstat == 'armstart':
             if lr[COL_START] is None:
                 self.log.info('Starting: ' + bib + ' @ ' + e.rawtime(1))
                 lr[COL_START] = e
+                self.__dorecalc = True	# starters flag recalc
             else:
                 self.log.error('Duplicate start rider = ' + bib
                                   + ' @ ' + str(e))
@@ -541,21 +671,25 @@ no.   rider                                   time  avg.')
             if self.timerstat == 'armfinish':
                 if lr[COL_RFTIME] is None:
                     lr[COL_RFTIME] = e
+                    self.__dorecalc = True	# finishers flag recalc
+                    glib.idle_add(self.query_rider, bib)
                 else:
                     self.log.error('Duplicate finish rider = ' + bib
                                       + ' @ ' + str(e))
-            glib.idle_add(self.query_rider, bib)
+
+    def recalculate(self):
+        """Update any internal structures and trigger export if required."""
+        pass
 
     def timeout(self):
-        """Poll for rfids and update elapsed time."""
+        """Update elapsed time and recalculate if required."""
         if not self.winopen:
             return False
-        e = self.meet.rfu.response()
-        while e is not None:
-            self.rfidtrig(e)
-            e = self.meet.rfu.response()
-        if self.finish is None and self.start is not None:
+        if self.start is not None:
             self.set_elapsed()
+        if self.__dorecalc:
+            self.__dorecalc = False
+            self.recalculate()
         return True
 
     def set_start(self, start='', lstart=None):
@@ -602,25 +736,30 @@ no.   rider                                   time  avg.')
     def set_running(self):
         """Update event status to running."""
         self.timerstat = 'running'
-        self.meet.rfu.arm()
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
 
     def set_finished(self):
         """Update event status to finished."""
         self.timerstat = 'finished'
-        self.meet.rfu.dearm()
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Finished')
         self.meet.stat_but.set_sensitive(False)
         if self.finish is None:
             self.set_finish(tod.tod('now'))
         self.set_elapsed()
 
+    def ridercat(self, cat):
+        """Return a category from the result for the riders cat."""
+        ret = ''        # default is the 'None' category - uncategorised
+        checka = cat.upper()
+        if checka in self.cats:
+            ret = checka
+        return ret
+
     def get_rider_elapsed(self, iter):
         """Return appropriate elapsed time for this rider."""
         ret = None
         st = self.riders.get_value(iter, COL_START)
         ft = self.riders.get_value(iter, COL_RFTIME)
-     
         if ft is not None:
             if st is not None:
                 if self.start is None:
@@ -689,8 +828,7 @@ no.   rider                                   time  avg.')
         self.event = event      # Note: now a treerowref
         self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
         self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
-        self.configpath = os.path.join(self.meet.configpath,
-                                       'event_' + self.evno)
+        self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('sportif')
         self.log.setLevel(logging.DEBUG)
@@ -703,15 +841,22 @@ no.   rider                                   time  avg.')
         self.finish = None
         self.winopen = True
         self.timerstat = 'idle'
-        self.minlap = tod.tod('20:00.0')
+        self.__dorecalc = False
+        self.cats = []
+        self.minmap = {}
+        self.maxmap = {}
+        self.mintime = tod.tod('5:00')		# abs min
+        self.maxtime = tod.tod('20h00:00')	# abs max
 
         self.riders = gtk.ListStore(gobject.TYPE_STRING, # BIB = 0
                                     gobject.TYPE_STRING, # NAMESTR = 1
                                     gobject.TYPE_STRING, # CAT = 2
                                     gobject.TYPE_STRING, # COMMENT = 3
-                                    gobject.TYPE_PYOBJECT, # START = 4
-                                    gobject.TYPE_PYOBJECT, # RFTIME = 5
-                                    gobject.TYPE_PYOBJECT) # RFSEEN = 6
+                                    gobject.TYPE_STRING, # SORTKEY = 4
+                                    gobject.TYPE_STRING, # TEAM = 5
+                                    gobject.TYPE_PYOBJECT, # START = 6
+                                    gobject.TYPE_PYOBJECT, # RFTIME = 7
+                                    gobject.TYPE_PYOBJECT) # RFSEEN = 8
 
         b = gtk.Builder()
         b.add_from_file(os.path.join(metarace.UI_PATH, 'sportif.ui'))
@@ -724,7 +869,6 @@ no.   rider                                   time  avg.')
         self.title_namestr = b.get_object('title_namestr')
         self.set_titlestr()
         self.time_lbl = b.get_object('time_lbl')
-        self.time_lbl.modify_font(pango.FontDescription("monospace bold"))
 
         # results pane
         t = gtk.TreeView(self.riders)
@@ -745,5 +889,4 @@ no.   rider                                   time  avg.')
         if ui:
             # connect signal handlers
             b.connect_signals(self)
-            self.meet.edb.editevent(event, winopen=True)
-            self.meet.rfu.arm()
+            self.meet.timer.setcb(self.rfidtrig)
