@@ -467,7 +467,7 @@ class timy(threading.Thread):
                 
                 # Write phase
                 if type(m) is tuple and type(m[0]) is str and m[0] in TCMDS:
-                    if m[0] == 'MSG' and not self.error:
+                    if m[0] == 'MSG' and self.port is not None and not self.error:
                         cmd = m[1]
                         self.log.debug('Sending rawmsg ' + repr(str(m[1])))
                         self.port.write(cmd)
