@@ -39,6 +39,7 @@ from metarace import scbwin
 from metarace import sender
 from metarace import uscbsrv
 from metarace import dbexport
+from metarace import rsync
 from metarace import timy
 from metarace import gemini
 from metarace import unt4
@@ -236,15 +237,20 @@ class trackmeet:
         self.running = False
         self.window.destroy()
 
-    ## Report printing support
-    def print_report(self, sections=[], subtitle='', docstr=''):
-        """Print the pre-formatted sections in a standard report."""
-        self.log.info('Printing report ' + repr(subtitle) + '...')
-
+    def default_template(self):
         tfile = os.path.join(self.configpath, 'template.ini')
         if not os.path.exists(tfile):
             tfile = None
+        return tfile
+
+    ## Report printing support
+    def print_report(self, sections=[], subtitle='', docstr='', prov=False, doprint=True):
+        """Print the pre-formatted sections in a standard report."""
+        self.log.info('Printing report ' + repr(subtitle) + '...')
+
+        tfile = self.default_template()
         rep = printing.printrep(template=tfile, path=self.configpath)
+        rep.set_provisional(prov)
         rep.strings['title'] = self.titlestr
         # subtitle should probably be property of meet
         rep.strings['subtitle'] = (self.subtitlestr + ' ' + subtitle).strip()
@@ -261,13 +267,21 @@ class trackmeet:
         ofile = os.path.join(self.configpath, 'output.pdf')
         with open(ofile, 'wb') as f:
             rep.output_pdf(f)
-        ofile = os.path.join(self.configpath, 'output.txt')
-        with open(ofile, 'wb') as f:
-            rep.output_text(f)
         ofile = os.path.join(self.configpath, 'output.xls')
         with open(ofile, 'wb') as f:
             rep.output_xls(f)
+        lb = ''
+        lt = []
+        if self.mirrorpath:
+            lb = os.path.join('/site', self.mirrorpath, 'output')
+            lt = ['pdf', 'xls']
+        ofile = os.path.join(self.configpath, 'output.txt')
+        with open(ofile, 'wb') as f:
+            rep.output_text(f, linkbase=lb, linktypes=lt)
         
+        if not doprint:
+            return False
+
         print_op = gtk.PrintOperation()
         print_op.set_print_settings(self.printprefs)
         print_op.set_default_page_setup(self.pageset)
@@ -508,16 +522,18 @@ class trackmeet:
         pass
 
     def menu_data_result_activate_cb(self, menuitem, data=None):
-        """Generate final result."""
+        """Export final result."""
+        provisional = self.provisional	# may be overridden below
         sections = []
         for e in self.edb:
             r = mkrace(self, e, False)
-            r.loadconfig()
-            report = r.result_report()
-            if len(report) > 0:
-                sections.extend(report)
+            if self.clubmode or r.evtype in ['classification']:
+                r.loadconfig()
+                report = r.result_report()
+                if len(report) > 0:
+                    sections.extend(report)
             r.destroy()	# is this valid??
-        self.print_report(sections, 'Results')
+        self.print_report(sections, 'Results', prov=provisional, doprint=False)
 
     def printprogram(self, prov=True):
         tmap = {}
@@ -572,9 +588,9 @@ class trackmeet:
                 for ri in cr:
                     evno = None
                     evname = None
-                    xtra1 = None
-                    xtra2 = None
-                    lenstr = None
+                    xtra1 = ''
+                    xtra2 = ''
+                    lenstr = ''
                     timestr = None
                     holders = 0
                     showcats = False
@@ -630,14 +646,16 @@ class trackmeet:
                 continue	# skip events that can't be in program?
             if h.evtype in ['flying 200', 'flying lap',
                                      'indiv tt', 'indiv pursuit',
-                                     'pursuit race']:
+                                     'pursuit race', 'team pursuit',
+                                      'team pursuit race']:
                 sec = printing.dual_ittt_startlist()
                 if h.evno in evmap:
+                    if 'race' not in h.evtype:
+                        sec.showheats = True
                     rmeta = evmap[h.evno]
                     sec.heading = rmeta[0]
-                    sec.lenstr = rmeta[1]
-                    sec.xtra1 = rmeta[2]
-                    sec.xtra2 = rmeta[3]
+                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            rmeta[2]]).strip()
                     if rmeta[5] > 0:	# placeholders - write in gaps
                         gapcount = rmeta[5]
                     showcats = rmeta[6]
@@ -658,9 +676,8 @@ class trackmeet:
                     rhold = [None, None, None, None, None, None]
                     rmeta = evmap[h.evno]
                     sec.heading = rmeta[0]
-                    sec.lenstr = rmeta[1]
-                    sec.xtra1 = rmeta[2]
-                    sec.xtra2 = rmeta[3]
+                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            rmeta[2]]).strip()
                     for heat in rmeta[9]:
                         sec.lines.append([heat, rhold, rhold, None])
             elif h.evtype == 'sprint final':	# best of 3 sprint final
@@ -670,13 +687,12 @@ class trackmeet:
                     rhold = [None, None, None, None, None, None]
                     rmeta = evmap[h.evno]
                     sec.heading = rmeta[0]
-                    sec.lenstr = rmeta[1]
-                    sec.xtra1 = rmeta[2]
-                    sec.xtra2 = rmeta[3]
+                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            rmeta[2]]).split()
                     for heat in rmeta[9]:
                         sec.lines.append([heat, rhold, rhold, None])
             else:
-                sec = printing.rmsevent()
+                sec = printing.twocol_startlist()
                 if h.evno in evmap:
                     rmeta = evmap[h.evno]
                     showcats = rmeta[6]
@@ -711,7 +727,7 @@ class trackmeet:
             h.destroy()
                 
         # Add team abbrevs
-        s = printing.rmsevent()
+        s = printing.twocol_startlist()
         s.heading = 'Abbreviations'
         s.lines = tlist
         if len(s.lines) > 0:
@@ -786,6 +802,16 @@ class trackmeet:
             self.log.info('Export complete.')
         except Exception as e:
             self.log.error('Error exporting data: ' + str(e))
+        glib.idle_add(self.mirror_start)
+
+    def mirror_start(self):
+        """Create a new rsync mirror thread unless in progress."""
+        if self.mirrorpath and self.mirror is None:
+            self.mirror = rsync.mirror(
+                localpath = os.path.join(self.configpath,'export',''),
+                remotepath = self.mirrorpath)
+            self.mirror.start()
+        return False	# for idle_add
 
     def menu_data_export_activate_cb(self, menuitem, data=None):
         """Export race data."""
@@ -811,6 +837,8 @@ class trackmeet:
                 series = self.edb.getvalue(e, eventdb.COL_SERIES)
                 evno = self.edb.getvalue(e, eventdb.COL_EVNO)
                 etype = self.edb.getvalue(e, eventdb.COL_TYPE)
+                prefix = self.edb.getvalue(e, eventdb.COL_PREFIX)
+                info = self.edb.getvalue(e, eventdb.COL_INFO)
                 key = evno	# no need to concat series, evno is unique
                 dirty = self.edb.getvalue(e, eventdb.COL_DIRTY)
                 if not dirty:   # check for any dependencies
@@ -821,7 +849,7 @@ class trackmeet:
                             break
                 if dirty:
                     dord.append(key)	# maintains ordering
-                    dmap[key] = [e, evno, etype, series]
+                    dmap[key] = [e, evno, etype, series, prefix, info]
             self.log.debug('Marked ' + str(len(dord)) + ' events dirty.')
   
             for k in dmap:	# only output dirty events
@@ -830,6 +858,7 @@ class trackmeet:
                 evno = dmap[k][1]
                 etype = dmap[k][2]
                 series = dmap[k][3]
+                evstr = (dmap[k][4] + ' ' + dmap[k][5]).strip()
                 self.edb.editevent(e, dirty=False)
                 r = mkrace(self, e, False)
                 r.loadconfig()
@@ -876,7 +905,7 @@ class trackmeet:
                 # starters
                 stset = []
                 stcount = 0
-                r.startlist_report()	# trigger rider model reorder
+                startrep = r.startlist_report()	# trigger rider model reorder
                 for result in r.result_gen():
                     bib = result[0]
                     first = ''
@@ -904,7 +933,32 @@ class trackmeet:
                 self.db.commit()
                 self.log.debug('Dirty event ' 
                                + repr(evno) + ' export complete.')
+
+                if self.mirrorpath and etype == 'classification': # ORFLAGGED?
+                    tfile = self.default_template()
+                    orep = printing.printrep(template=tfile,
+                                             path=self.configpath)
+                    orep.strings['title'] = self.titlestr
+                    # subtitle should probably be property of meet
+                    orep.strings['subtitle'] = evstr
+                    orep.strings['datestr'] = strops.promptstr('Date:', self.datestr)
+                    #orep.strings['commstr'] = strops.promptstr('Chief Commissaire:',
+                                                  #self.commstr)
+                    #orep.strings['orgstr'] = strops.promptstr('Organiser: ', self.orgstr)
+                    orep.strings['diststr'] = self.locstr
+                    orep.strings['docstr'] = evstr + ' Classification'
+                    orep.set_provisional(self.provisional)
+                    # update files and trigger mirror
+                    outsec = r.result_report()
+                    for sec in outsec:
+                        orep.add_section(sec)
+                    ofile = os.path.join(self.configpath, 'export',
+                                         'event_' + str(evno) + '.txt')
+                    with open(ofile, 'wb') as f:
+                        orep.output_text(f)	# with links?
+
                 r.destroy()
+            glib.idle_add(self.mirror_start)
             self.log.info('Race info export.')
         except Exception as e:
             self.log.error('Error exporting results: ' + str(e))
@@ -1091,6 +1145,11 @@ class trackmeet:
         else:
             tt = tod.tod('now')
             self.clock_label.set_text(tt.meridian())
+
+            # check for completion in the rsync module
+            if self.mirror is not None:
+                if not self.mirror.is_alive():
+                    self.mirror = None
             #if type(self.scbwin) is scbwin.scbclock:
                 #self.gemini.ctick(tt)	# dubious
             #self.announce.postxt(0,72,tt)
@@ -1208,6 +1267,7 @@ class trackmeet:
     def shutdown(self, msg):
         """Cleanly shutdown threads and close application."""
         self.started = False
+            
         self.db.exit(msg)
         self.gemini.exit(msg)
         self.scb.exit(msg)
@@ -1215,6 +1275,10 @@ class trackmeet:
         self.main_timer.exit(msg)
         self.backup_timer.exit(msg)
         print ('Waiting for workers to exit...')
+        if self.mirror is not None:
+            print('\tlive result mirror.')
+            self.mirror.join()
+            self.mirror = None
         print('\tdatabase export.')
         self.db.join()
         print('\tauxilliary scoreboard.')
@@ -1299,6 +1363,7 @@ class trackmeet:
         cw.set('meet', 'organiser', self.orgstr)
         cw.set('meet', 'commissaire', self.commstr)
         cw.set('meet', 'logos', self.logos)
+        cw.set('meet', 'mirrorpath', self.mirrorpath)
         cw.set('meet', 'clubmode', self.clubmode)
         cw.set('meet', 'showevno', self.showevno)
         cw.set('meet', 'provisional', self.provisional)
@@ -1338,6 +1403,7 @@ class trackmeet:
                                         'commissaire':'',
                                         'logos':'',
                                         'curevent':'',
+                                        'mirrorpath':'',
                                         'tracklayout':'DISC',
                                         'provisional':'No',
                                         'id':''})
@@ -1422,6 +1488,7 @@ class trackmeet:
         self.orgstr = cr.get('meet', 'organiser')
         self.commstr = cr.get('meet', 'commissaire')
         self.logos = cr.get('meet', 'logos')
+        self.mirrorpath = cr.get('meet', 'mirrorpath')
         self.set_title()
 
         # result options
@@ -1536,6 +1603,7 @@ class trackmeet:
         self.docindex = 0	# HACK: use for session number
         self.lastexport = None	# timestamp of last db dump
         self.exportpending = False
+        self.mirrorpath = ''	# default rsync mirror path
 
         # printer preferences
         paper = gtk.paper_size_new_custom('metarace-full',
@@ -1563,6 +1631,7 @@ class trackmeet:
         self.gemport = ''
         self.db = dbexport.dbexport()	# hack for CA Track Nats-> to be incl
         self.dbport = ''
+        self.mirror = None
 
         b = gtk.Builder()
         b.add_from_file(os.path.join(metarace.UI_PATH, 'trackmeet.ui'))
