@@ -1,4 +1,3 @@
-## TODO: modify wheeltime to report via glib_idle_add callback
 
 # Metarace : Cycle Race Abstractions
 # Copyright (C) 2012  Nathan Fraser
@@ -21,20 +20,30 @@
 This module provides a thread object which interfaces with the
 Times-7 Wheeltime RFID system. Once connected, the thread will
 collect RFID events from the wheeltime and deliver them to a calling
-thread via a response queue.
-
-A calling thread creates a wheeltime thread and then polls for 
-new RFIDs with the wheeltime.response() method.
+thread via a specified callback.
 
 TCP/IP communicaton with an attached wheeltime unit is handled
 by blocking I/O in a sub thread.
 
+  Messages are returned as TOD objects:
+
+        index:          power:batt      99:0 or '' for trig/sts
+        chan:           BOX     rfid
+                        STA     n/a
+                        MAN     manual trigger
+                        STS     status n/a
+        timeval:        time of day of passing/trig/status
+        refid:          transponder id with leading zeros stripped
+			and T7 prefix stripped
+
+  Sent to mainloop via glib.idle_add of the provided callback
+
+
 """
 
 import threading
 import Queue
 import logging
-import decimal
 import socket
 import time
 import glib
@@ -42,15 +51,15 @@ import glib
 from metarace import tod
 
 # System defaults
-#WHEELIP = 'localhost'		# Testing address
-WHEELIP = '192.168.95.32'	# CSV wheeltime IP
-#WHEELIP = '10.19.1.103'		# Old Times-7 wheeltime IP addr
-WHEELRAWPORT = 10000		# Port for raw tag stream
-WHEELFSPORT = 10200		# Port for FS/LS filtered stream
-WHEELCMDPORT = 9999		# Wheeltime command port
+TAGPREFIX = '058001'		# Wheeltime rfid prefix
+DEFPORT = '192.168.95.32'	# CV wheeltime IP
+WHEELRAWPORT = 10000		# TCP Port for raw tag stream
+WHEELFSPORT = 10200		# TCP Port for FS/LS filtered stream (normal)
+WHEELCMDPORT = 9999		# TCP Wheeltime command port (defunct?)
+WHEELPHOTOTHRESH = tod.tod('0.1')	# Wheeltime confidence ~0.1s
 
 # thread queue commands -> private to timy thread
-TCMDS = ('RFID', 'EXIT', 'ADDR', 'MSG')
+TCMDS = ('RFID', 'EXIT', 'PORT', 'TRIG', 'REPL', 'MSG')
 
 # Logging defaults
 RFID_LOG_LEVEL = 16	# lower so not in status and on-screen logger.
@@ -104,7 +113,7 @@ class wtio(threading.Thread):
         self.running = False
 
     def readline(self, s=None):
-        """Return newline delimited lines from socket.
+        """Return whole lines from socket.
 
         This function works on an input buffer, returning one complete
         line per call or None if one is not yet fully received.
@@ -133,12 +142,15 @@ class wtio(threading.Thread):
             lrc = int(s[34:36], 16)
             if sum == lrc:
                 #tagid=s[4:16]	## NOTE: Using 'shortform' tag ids
-                if s[4:10] == '058001':	# match id prefix
+                if s[4:10] == TAGPREFIX:	# match id prefix
                     tagid=(s[10:16]).lower()
                     timestr = '{0}:{1}:{2}.{3:02}'.format(s[26:28], s[28:30],
                                    s[30:32], int(s[32:34], 16))
+                    istr = '00:0'	# emulate THbC info, but zero for WT
+                    chanstr = 'BOX'
                     self.cqueue.put_nowait(('RFID',
-                          tod.tod(timestr, 'RFID', '', tagid)))
+                                            tod.tod(timestr, index=istr,
+                                                 chan=chanstr, refid=tagid)))
                 else:
                     self.log.warn('Spurious tag id: ' + s[4:10] + ' :: ' 
                                     + s[10:16])
@@ -147,16 +159,17 @@ class wtio(threading.Thread):
                                + hex(sum) + ' != ' + hex(lrc))
         elif len(s) == 30 and s[0:8] == 'ab010a2c':
             # Process a trigger event
-            ## HACK : Temporary change to use mainloop callback until
-            ##        can replace with newer gObject signal based handlers?
             sum = ipico_lrc(s, 28)
             lrc = int(s[28:30], 16)
             if sum == lrc:
                 timestr = '{0}:{1}:{2}.{3:02}'.format(s[16:18], s[18:20],
                                s[20:22], int(s[22:24], 16))
-                self.cqueue.put_nowait(('TRIG',
-                      tod.tod(timestr, 'RFID', '', 'trig')))
-                #self.log.debug('TRIG MSG: ' + repr(s))
+                istr = '00:0'
+                chanstr = 'MAN'
+                tagid = ''	# emulate THbC manual trigger (lstripped)
+                self.cqueue.put_nowait(('RFID',
+                                        tod.tod(timestr, index=istr,
+                                                 chan=chanstr, refid=tagid)))
             else:
                 self.log.warn('Incorrect char sum message skipped: ' 
                                + hex(sum) + ' != ' + hex(lrc))
@@ -182,36 +195,47 @@ class wtio(threading.Thread):
         except Exception as e:
             self.running = False
             self.log.error('WTIO Exception: ' + repr(e))
-            #raise		# No need to except here - thread dies.
 
 class wheeltime(threading.Thread):
     """Wheeltime thread object class."""
-    def __init__(self, addr=None, name=None):
+    def __init__(self, port=None, name='wheeltime'):
         """Construct wheeltime thread object.
 
         Named parameters:
  
-          addr -- ip address or hostname of wheeltime unit
+          port -- ip address or hostname of wheeltime unit
           name -- text identifier for use in log messages
 
         """
 
         threading.Thread.__init__(self) 
-        nms = ['wheeltime']
-        if name is not None:
-            nms.append(str(name))
-        self.name = '.'.join(nms)
+        self.name = name
         self.addr = None
-        self.armed = False
         self.cqueue = Queue.Queue()	# command queue
-        self.rqueue = Queue.Queue()	# response queue
         self.log = logging.getLogger(self.name)
         self.log.setLevel(logging.DEBUG)
+        self.setcb()
         self.io = None
-        self.trigcb = None
         self.running = False
-        if addr is not None:
-            self.setaddr(addr)
+        if port is not None:
+            self.setport(port)
+
+    def photothresh(self):
+        """Return the relevant photo finish threshold."""
+        return WHEELPHOTOTHRESH
+
+    def __defcallback(self, evt=None):
+        """Default callback is a tod log entry."""
+        self.log.debug(str(evt))
+        return False
+
+    def setcb(self, func=None):
+        """Set or clear the event callback."""
+        # if func is not callable, gtk mainloop will catch the error
+        if func is not None:
+            self.__cb = func
+        else:
+            self.__cb = self.__defcallback
 
     def clrmem(self):
         """Clear wheeltime memory."""
@@ -226,19 +250,19 @@ class wheeltime(threading.Thread):
         self.running = False
         self.cqueue.put_nowait(('EXIT', msg))
 
-    def setaddr(self, addr=None):
+    def setport(self, device=None):
         """Request new wheeltime device address."""
-        self.cqueue.put_nowait(('ADDR', addr))
+        self.cqueue.put_nowait(('PORT', device))
 
-    def arm(self):
-        """Arm response queue."""
-        self.log.debug('Arm response queue')
-        self.armed = True
+    def sane(self):
+        """Nothing to do with wheeltime."""
+        pass
 
-    def dearm(self):
-        """De-arm response queue."""
-        self.log.debug('De-arm response queue')
-        self.armed = False
+    def stop_session(self):
+        pass
+
+    def start_session(self):
+        pass
 
     def sync(self):
         """Rough synchronise wheeltime RTC to PC."""
@@ -247,31 +271,20 @@ class wheeltime(threading.Thread):
                      str(t[0])[2:], t[1], t[2], t[6], t[3], t[4], t[5])
         self.cqueue.put_nowait(('MSG', 'ab000701' + datestr + '\n'))
 
-    def trig(self, nt=None):
-        """Insert a fake trigger event."""
-        t = tod.tod('now', 'FAKE', refid='trig')
-        if nt is not None:
-            t.timeval = nt.timeval
+    def trig(self, timeval='now', index='FAKE', chan='MAN', refid='0'):
+        """Create a fake timing event.
+
+           Generate a new tod object to mimic a message as requested
+           and pipe it to the command thread. Default tod is the
+           'now' time in the calling thread.
+
+        """
+        t = tod.tod(timeval, index, chan, refid.lstrip('0'))
         self.cqueue.put_nowait(('TRIG', t))
 
-    def replay(self, srcfile=None):
-        """Replay an RFID log file into response queue."""
-        with open(srcfile, 'rb') as f:
-            for l in f:
-                rvec = l.split()
-                if len(rvec) == 6 and rvec[3] == 'RFID':
-                    self.rqueue.put_nowait(tod.tod(rvec[-2], 'RFiD',
-                                                   '', rvec[-1]))
-                       
-    def response(self):
-        """Check for RFID events in response queue."""
-        r = None
-        try:
-            r = self.rqueue.get_nowait()
-            self.rqueue.task_done()
-        except Queue.Empty:
-            r = None
-        return r
+    def replay(self, filename=''):
+        """Read passings from file and process."""
+        self.cqueue.put_nowait(('REPL', filename))
 
     def wait(self):		# NOTE: Do not call from cmd thread
         """Suspend calling thread until cqueue is empty."""
@@ -280,55 +293,51 @@ class wheeltime(threading.Thread):
     def command(self, command):		# NOTE: Lazy!
         """Connect to serv and dump a command."""
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        s.settimeout(0.5)
+        s.settimeout(1.0)
         s.connect((self.addr, WHEELCMDPORT))
         sendall(s, command.encode('latin_1'))
         s.shutdown(socket.SHUT_RDWR)
         s.close()
 
+    def status(self):
+        if self.connected():
+            self.log.info('Wheeltime unit connected on: ' + repr(self.addr))
+        else:
+            self.log.info('Wheeltime not connected.')
+
     def connected(self):
         """Return True if wheeltime unit connected."""
         return self.io and self.io.running
 
-    def get_trig_cb(self):
-        """Fetch current trigger callback."""
-        return self.trigcb
-
-    def set_trig_cb(self, cb=None):
-        """Set or clear the trigger callback function."""
-        self.trigcb = cb
-
     def run(self):
         """Called via threading.Thread.start()."""
         self.running = True
-        #self.log.debug('Starting')
+        self.log.debug('Starting')
         while self.running:
             try:
-                # Read Phase
+                # Read Phase - all msgs come through cqueue in wheeltime
                 m = self.cqueue.get()
                 self.cqueue.task_done()
                 
                 # Write phase
                 if m[0] == 'RFID':
-                    assert type(m[1]) is tod.tod
                     self.log.log(RFID_LOG_LEVEL, ' ' + str(m[1]))
-                    if self.armed:
-                        self.rqueue.put_nowait(m[1])
-                        #self.log.debug('Queueing RFID: ' + str(m[1]))
+                    glib.idle_add(self.__cb, m[1])
                 elif m[0] == 'TRIG':
-                    assert type(m[1]) is tod.tod
-                    self.log.log(RFID_LOG_LEVEL, ' ' + str(m[1]))
-                    if self.trigcb is not None:
-                        glib.idle_add(self.trigcb, m[1])
+                    if type(m[1]) is tod.tod:
+                        self.log.log(RFID_LOG_LEVEL, str(m[1]))
+                        glib.idle_add(self.__cb, m[1])
                 elif m[0] == 'MSG':
                     if self.connected():
                         self.command(m[1])
                     else:
                         self.log.warn('Wheeltime not connected.')
+                elif m[0] == 'REPL':
+                    self.log.warn('Wheeltime replay not implemented.')
                 elif m[0] == 'EXIT':
                     self.running = False
                     self.log.debug('Request to close : ' + str(m[1]))
-                elif m[0] == 'ADDR':
+                elif m[0] == 'PORT':
                     self.addr = None
                     if self.io is not None:
                         self.io.close()
@@ -347,33 +356,30 @@ class wheeltime(threading.Thread):
                 self.log.error('Exception: ' + str(type(e)) + str(e))
         if self.io is not None:
             self.io.close()
-        self.trigcb = None
+        self.setcb()
         self.log.info('Exiting')
 
 if __name__ == "__main__":
-    w = wheeltime('localhost')
+    import metarace
+    import gtk
+    import random
+    metarace.init()
+    w = wheeltime(DEFPORT)
     lh = logging.StreamHandler()
     lh.setLevel(logging.DEBUG)
     lh.setFormatter(logging.Formatter(
                     "%(asctime)s %(levelname)s:%(name)s: %(message)s"))
     w.log.addHandler(lh)
-    w.start()
-    w.clrmem()
-    w.sync()
     try:
+        w.start()
+        #w.clrmem()
+        #w.sync()
         w.wait()
-        w.arm()
-        for r in ['fake01', 'fake02', 'fake03', 'fake04']:
-            w.trig(r)
-            time.sleep(5)
-            # collect any events from response queue
-            e = w.response()
-            while e is not None:
-                print ("RFID Event: " + str(e))
-                e = w.response()
-        w.dearm()
+        gtk.main()
     except:
+        w.wait()
         w.exit('Exception')
+        w.join()
         raise
     w.exit('Complete')
     w.join()
